indicators_example_request = """{
   "queryTerm":"/Users/mknopf/code/test.sh",
   "enclaveGuids":[
      "3a93fab3-f87a-407a-9376-8eb3fae99b4e"
   ],
   "priorityScores":[
      3
   ],
   "types":[
      "SOFTWARE"
   ],
   "from":1596607968000,
   "to":1598308171000,
   "sortColumn":"PROCESSED_AT",
   "attributes":[
      {
         "type":"THREAT_ACTOR",
         "value":"BAD PANDA"
      }
   ],
   "cursor":"eyJwYWdlTnVtYmVyIjoxLCJwYWdlU2l6ZSI6Miwib2Zmc2V0Ijo0fQ=="
}"""

indicators_submission_example_request = """
{
  "title": "Report, complex test",
  "content": {
    "indicators": [
      {
        "observable": {
          "value": "verybadurl",
          "type": "URL"
        },
        "validFrom": 1604510497000,
        "validTo": 1607102497000,
        "maliciousScore": "BENIGN",
        "confidenceScore": "LOW",
        "attributes": [
          {
            "entity": {
              "value": "ActorName",
              "type": "THREAT_ACTOR"
            },
            "validFrom": 1604510497000,
            "validTo": 1607102497000,
            "confidenceScore": "LOW"
          },
          {
            "entity": {
              "value": "MalwareName",
              "type": "MALWARE"
            },
            "validFrom": 1604510497000,
            "validTo": 1607102497000,
            "confidenceScore": "MEDIUM"
          }
        ],
        "relatedObservables": [
          {
            "entity": {
              "value": "2.2.2.2",
              "type": "IP4"
            },
            "validFrom": 1604510497000,
            "validTo": 1607102497000,
            "confidenceScore": "LOW"
          },
          {
            "entity": {
              "value": "wwww.relatedUrl.com",
              "type": "URL"
            },
            "validFrom": 1604510497000,
            "validTo": 1607102497000,
            "confidenceScore": "HIGH"
          }
        ],
        "properties": {
          "propertyKey": "propertyValue"
        },
        "tags": [
          "importantTag",
          "anotherTag"
        ]
      }
    ]
  },
  "enclaveGuid": "c0f07a9f-76e4-48df-a0d4-c63ed2edccf0",
  "externalId": "external-1234",
  "externalUrl": "externalUrlValue",
  "timestamp": 1607102497000,
  "tags": ["random_tag"],
  "rawContent": "blob of text"
}
"""

non_structured_submission_example_request = """
{
  "title": "Report, complex test",
  "content": "MALICIOUS IP: 8.8.8.8",
  "enclaveGuid": "c0f07a9f-76e4-48df-a0d4-c63ed2edccf0",
  "externalId": "external-1234",
  "externalUrl": "externalUrlValue",
  "timestamp": 1607102497000,
  "tags": ["random_tag"]
}
"""


safelist_summaries = """
[
  {
    "guid": "test-library-guid-1",
    "name": "test-library-name-1",
    "companyGuid": "test-company-guid-1",
    "excerpt": "",
    "createdAt": 1618258235178,
    "updatedAt": 1618258235331,
    "createdBy": "test-user-1@trustar.co",
    "updatedBy": "test-user-1@trustar.co"
  },
  {
    "guid": "test-library-guid-2",
    "name": "test-library-name-2",
    "companyGuid": "test-company-guid-2",
    "excerpt": "",
    "createdAt": 1616791794869,
    "updatedAt": 1618288545872,
    "createdBy": "test-user-2@trustar.co",
    "updatedBy": "test-user-2@trustar.co"
  }
]
"""

safelist_details = """
{
  "guid": "test-library-guid-1",
  "name": "test-library-name-1",
  "companyGuid": "test-company-guid-1",
  "excerpt": "",
  "createdAt": 1618258235178,
  "updatedAt": 1618258235331,
  "createdBy": "test-user-1@trustar.co",
  "updatedBy": "test-user-1@trustar.co",
  "entries": [
      {
          "guid": "entry-guid-1",
          "entity": "good-email@test-domain.com",
          "type": "EMAIL_ADDRESS",
          "createdBy": "test-user-1@trustar.co",
          "createdAt": 1618288545871
      }
  ]
}
"""

entities_extraction = """
[
  {
      "entity": "8.8.8.8",
      "type": "IP4"
  },
  {
      "entity": "good-email@test-domain.com",
      "type": "EMAIL_ADDRESS"
  }
]
"""

enclaves = """
[
  {
      "name": "test_name_1",
      "templateName": "Private Enclave",
      "workflowSupported": false,
      "read": true,
      "create": true,
      "update": true,
      "id": "test-id-1",
      "type": "INTERNAL"
  },
  {
      "name": "test_name_2",
      "templateName": "Private Enclave",
      "workflowSupported": false,
      "read": true,
      "create": true,
      "update": true,
      "id": "test-id-2",
      "type": "INTERNAL"
  }
]
"""

serialized_workflow_config = """
{
  "type": "INDICATOR_PRIORITIZATION",
  "priorityScores": ["MEDIUM", "HIGH"],
  "observableTypes": ["URL", "IP4", "IP6", "SHA256"],
  "workflowSource": {
    "enclaveSourceConfig": [
      {
        "enclaveGuid": "test-enclave-id",
        "weight": 3
      },
      {
        "enclaveGuid": "test-enclave-id2",
        "weight": 3
      },
      {
        "enclaveGuid": "test-enclave-id3",
        "weight": 1
      },
      {
        "enclaveGuid": "test-enclave-id4",
        "weight": 5
      }
    ]
  },
  "workflowDestination": {
    "enclaveDestinationConfigs": [
      {
        "enclaveGuid": "test-enclave-id",
        "destinationType": "ENCLAVE"
      }
    ]
  }
}
"""

observables_search_example_request = """{
    "queryTerm": "query",
    "from":1596607968000,
    "to":1598308171000,
    "sortColumn": "FIRST_SEEN",
    "sortOrder": "ASC",
    "enclaveGuids": ["4bdc3f5b-3ed5-4d99-b20c-2d801866ef0b"],
    "types": ["MD5"]
}"""



from __future__ import absolute_import

from trustar2.handlers import (
    SearchIndicator, 
    Submission, 
    Safelist, 
    TagIndicator,
    TagSubmission, 
    Workflow,
    Account, 
    Observables
)

from trustar2.trustar import TruStar



from __future__ import unicode_literals

from trustar2.trustar_enums import TSEnum
from trustar2.handlers.search_handler import SearchHandler
import pytest


TEST_DATE = [1583960400000, "2020-03-11T21:00:00"]

@pytest.fixture
def search_handler(ts):
    return SearchHandler(ts)

def test_set_query_term(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_query_term("TEST_TERM")
    values = [param.value for param in search_handler.payload_params]
    assert len(search_handler.payload_params) == 1
    assert values[0] == "TEST_TERM"


def test_override_query_term(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_query_term("TEST_TERM1")
    assert search_handler.payload_params.get("queryTerm") == "TEST_TERM1"
    assert len(search_handler.payload_params) == 1
    search_handler.set_query_term("TEST_TERM2")
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("queryTerm") == "TEST_TERM2"

@pytest.mark.parametrize("from_date", TEST_DATE)
def test_set_from(search_handler, from_date):
    assert len(search_handler.payload_params) == 0
    search_handler.set_from(from_date)
    assert search_handler.payload_params.get("from") == TEST_DATE[0]
    assert len(search_handler.payload_params) == 1


def test_set_from_fail(search_handler):
    assert len(search_handler.payload_params) == 0
    with pytest.raises(TypeError):
        search_handler.set_from("XXXX-XX-XX")
    assert len(search_handler.payload_params) == 0


def test_invalid_dates(search_handler):
    search_handler.set_from("1 day ago")
    search_handler.set_to("2 days ago")
    with pytest.raises(AttributeError):
        search_handler._validate_dates()


@pytest.mark.parametrize("to_date", TEST_DATE)
def test_set_to(search_handler, to_date):
    assert len(search_handler.payload_params) == 0
    search_handler.set_to(to_date)
    assert search_handler.payload_params.get("to") == TEST_DATE[0]
    assert len(search_handler.payload_params) == 1


def test_set_to_fail(search_handler):
    assert len(search_handler.payload_params) == 0
    with pytest.raises(TypeError):
        search_handler.set_to("XXXX-XX-XX")
    assert len(search_handler.payload_params) == 0


def test_set_enclave_ids(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_enclave_ids(["TEST_ENCLAVE_ID"])
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("enclaveGuids") == ["TEST_ENCLAVE_ID"]

def test_set_enclave_ids_single_value(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_enclave_ids("TEST_ENCLAVE_ID")
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("enclaveGuids") == ["TEST_ENCLAVE_ID"]

def test_set_included_tags(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_included_tags(["test-tag"])
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("includedTags") == ["test-tag"]

def test_set_included_tags_repeated_values(search_handler):
    tags = ['a', 'b', 'c', 'a' , 'b' , 'd']
    expected_tags = ['a', 'b', 'c', 'd']
    assert len(search_handler.payload_params) == 0
    search_handler.set_included_tags(tags)
    assert(len(search_handler.payload_params)) == 1
    assert(sorted(search_handler.payload_params.get("includedTags"))) == expected_tags

def test_set_excluded_tags(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_excluded_tags(["test-tag"])
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("excludedTags") == ["test-tag"]

def test_set_excluded_tags_repeated_values(search_handler):
    tags = ['a', 'b', 'c', 'a' , 'b' , 'd']
    expected_tags = ['a', 'b', 'c', 'd']
    assert len(search_handler.payload_params) == 0
    search_handler.set_excluded_tags(tags)
    assert(len(search_handler.payload_params)) == 1
    assert(sorted(search_handler.payload_params.get("excludedTags"))) == expected_tags

def test_set_sort_column_with_default_options(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_sort_column("UPDATED")
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("sortColumn") == "UPDATED"

def test_set_sort_column_with_invalid_options(search_handler):
    class Options(TSEnum):
        OPT = "OPT"
    assert len(search_handler.payload_params) == 0
    with pytest.raises(AttributeError):
        search_handler.set_sort_column("UPDATED", Options)
    assert len(search_handler.payload_params) == 0

def test_set_sort_column_with_valid_options(search_handler):
    class Options(TSEnum):
        OPT = "OPT"
    assert len(search_handler.payload_params) == 0
    search_handler.set_sort_column("OPT", Options)
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("sortColumn") == "OPT"

def test_set_sort_order_with_valid_options(search_handler):
    assert len(search_handler.payload_params) == 0
    search_handler.set_sort_order("ASC")
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("sortOrder") == "ASC"
    search_handler.set_sort_order("DESC")
    assert len(search_handler.payload_params) == 1
    assert search_handler.payload_params.get("sortOrder") == "DESC"

def test_set_sort_order_with_invalid_options(search_handler):
    assert len(search_handler.payload_params) == 0
    with pytest.raises(AttributeError):
        search_handler.set_sort_order("ASCC")
    assert len(search_handler.payload_params) == 0


def test_set_page_size(search_handler):
    assert len(search_handler.query_params) == 0
    search_handler.set_page_size(999)
    assert len(search_handler.query_params) == 1
    assert search_handler.query_params.get("pageSize") == 999



class Base(object):

    TYPE_ATTRIBUTES = ("_entity_type", "workflow_type")

    @staticmethod
    def _get_camelcase(attribute):
        if attribute in Base.TYPE_ATTRIBUTES:
            return "type"

        tmp = attribute.split("_")
        return tmp[0] + "".join([w.title() for w in tmp[1:]])

    @staticmethod
    def _get_serialized(attribute_value):
        serializable = getattr(attribute_value, "serialize", None)
        if serializable:
            attribute_value = attribute_value.serialize()

        return attribute_value

    def _get_serialized_attribute(self, attribute_value):
        if isinstance(attribute_value, list):
            attribute_value = [self._get_serialized(a) for a in attribute_value]

        else:
            attribute_value = self._get_serialized(attribute_value)

        return attribute_value

    def serialize(self):
        return {self._get_camelcase(k): self._get_serialized_attribute(v)
                for k, v in self.__dict__.items()
                if v is not None
                }


class Entity(Base):

    def __init__(self, value):
        self.value = value



from math import ceil
import requests
from requests.exceptions import HTTPError
import time

from .log import get_logger


logger = get_logger(__name__)


class ApiClient(object):
    """
    This class is used to make HTTP requests to the TruStar API.
    """

    INVALID_TOKEN_MESSAGES = (
        "Expired oauth2 access token",
        "Invalid oauth2 access token",
    )

    def __init__(self, config):
        self.config = config
        self.token = None
        self.proxy = config.get_proxy()


    def _refresh_token(self):
        """
        Refreshes oauth token.
        """
        logger.debug("Authenticating")
        client_auth = requests.auth.HTTPBasicAuth(
            self.config.api_key, self.config.api_secret
        )
        post_data = {"grant_type": "client_credentials"}
        endpoint = self.config.request_details.get("auth_endpoint")
        response = requests.post(
            endpoint, auth=client_auth, data=post_data, verify=True, proxies=self.proxy
        )
        response.raise_for_status()
        self.token = response.json()["access_token"]

    def _get_token(self):
        """
        Retrieves token.
        :returns: oauth token.
        """
        if self.token is None:
            self._refresh_token()

        return self.token

    def _token_is_expired(self, response):
        """
        Checks if an HTTP response failed due to a expired oauth token.

        :param response: HTTP response object.
        :returns: True or False indicating if the response failed.
        """
        if response.status_code != 400:
            return False

        body = response.json()
        return str(body.get("error_description")) in self.INVALID_TOKEN_MESSAGES

    def _get_headers(self, method):
        """
        Forms the HTTP headers according to user's config and method.

        :param method: HTTP method.
        :returns: HTTP headers as dict.
        """
        headers = {
            "Authorization": "Bearer " + self._get_token(),
            "Client-Metatag": self.config.client_metatag,
        }

        client_type = self.config.request_details.get("client_type")
        client_version = self.config.request_details.get("client_version")

        if client_type:
            headers["Client-Type"] = client_type

        if client_version:
            headers["Client-Version"] = client_version

        if method in ("POST", "PUT"):
            headers["Content-Type"] = "application/json"

        return headers

    def _request(self, method, endpoint, payload=None, params=None):
        """
        Generic request method to handle diffent HTTP requests.

        :param method: HTTP request method.
        :param endpoint: URL to be requested.
        :param payload: payload to include in the request.
        :param params: params to include in the request.

        :returns: response object.
        """
        retry = self.config.request_details.get("retry")
        attempted = False

        while not attempted or retry:
            headers = self._get_headers(method)
            response = requests.request(
                method=method,
                url=endpoint,
                headers=headers,
                json=payload,
                params=params,
                proxies=self.proxy,
            )
            if response.status_code == requests.codes.ok:
                return response

            if self._token_is_expired(response):
                self._refresh_token()

            elif retry and response.status_code == requests.codes.too_many:
                retry = self._sleep(response)

            else:
                message = "{} {} Error (Trace-Id: {})".format(
                    response.status_code,
                    "Client" if response.status_code < 500 else "Server",
                    self._get_trace_id(response),
                )
                raise HTTPError(request=message, response=response)

    def _sleep(self, response):
        """
        Sleeps if response is a 429 and wait time is lower the max_wait_time in config.

        :param response: HTTP response object.
        :returns: True or False indicating if it is necessary to keep trying.
        """
        wait_time = ceil(response.json().get("waitTime") / 1000)
        logger.debug("Waiting {} seconds until next request allowed.".format(wait_time))
        keep_trying = wait_time <= self.config.request_details.get("max_wait_time")
        if keep_trying:
            time.sleep(wait_time)

        return keep_trying

    @staticmethod
    def _get_trace_id(response):
        """
        Fetches the trace id from the HTTP response header.

        :param response: HTTP response object.
        :returns: TruSTAR Trace-Id if found.
        """
        trace_id = response.headers.get("Trace-Id")
        return trace_id if trace_id is not None else None

    def fetch(self, query, use_empty_payload=False):
        payload = query.params.serialize() if query.params is not None else None
        if not payload and not use_empty_payload:
            payload = None

        return self._request(
            query.method.name, query.endpoint, payload, params=query.query_string
        )



from __future__ import unicode_literals

from trustar2.base import fluent, Methods, ParamsSerializer, Param, get_timestamp
from trustar2.handlers.base_handler import BaseHandler
from trustar2.query import Query
from trustar2.trustar_enums import ObservableTypes


@fluent
class Safelist(BaseHandler):

    summaries = "/safelist-libraries"
    details =  summaries + "/{}"
    extract = summaries + "/extract"

    def __init__(self, trustar_config=None):
        super(Safelist, self).__init__(trustar_config)
        self.library_guid = None


    @property
    def summaries_endpoint(self):
        return self.config.request_details.get("api_endpoint") + self.summaries


    @property
    def details_endpoint(self):
        return self.config.request_details.get("api_endpoint") + self.details.format(self.library_guid)


    @property
    def extract_endpoint(self):
        return self.config.request_details.get("api_endpoint") + self.extract


    def set_library_guid(self, library_guid):
        self.library_guid = library_guid


    def _verify_entry(self, entry):
        entity = entry.get("entity")
        entity_type = entry.get("type")
        if not entity or not entity_type:
            raise AttributeError("A new entry should have 'entity' and 'type' fields")

        if not isinstance(entity, type("")):
            raise AttributeError("Your entity value should be a string")

        obs_types = ObservableTypes.members()
        if entity_type not in obs_types:
            msg = "Entity type should be one of the following: {}".format(obs_types)
            raise AttributeError(msg)


    def set_safelist_entries(self, entries):
        if isinstance(entries, dict):
            entries = [entries]
        
        for entry in entries:
            self._verify_entry(entry)

        self.set_payload_param("entries", entries)


    def set_library_name(self, library_name):
        if not isinstance(library_name, type("")):
            raise AttributeError("Library name must be a string.")

        self.set_payload_param("name", library_name)

    
    def set_text_to_be_extracted(self, text):
        if not isinstance(text, type("")):
            raise AttributeError("You can only submit a text for extraction.")

        self.set_payload_param("text", text)


    def _validate_library_guid_is_present(self):
        if self.library_guid is None:
            raise AttributeError("No library guid was found.")


    def get_safelist_libraries(self):
        """Retrieves safelist details given a library guid. 

        You have to call 'set_library_guid' before calling this method.

        :returns: HTTP response with safelist library summaries in it's content.
        """
        return Query(self.config, self.summaries_endpoint, Methods.GET).set_params(self.payload_params).execute()
    

    def get_safelist_details(self):
        """Retrieves safelist details given a library guid. 

        You have to call 'set_library_guid' before calling this method.

        :returns: HTTP response with Safelist Library Details in it's content.
        """
        self._validate_library_guid_is_present()
        return Query(self.config, self.details_endpoint, Methods.GET).set_params(self.payload_params).execute()


    def create_entries(self):
        """Creates a new entry in a safelist library.

        You have to call 'set_safelist_entries' and 'set_library_guid' 
        before calling this method.

        :returns: HTTP response with Safelist Library Details in it's content.
        """
        self._validate_library_guid_is_present()
        if not self.payload_params.get("entries"):
            raise AttributeError(
                "You must call the 'set_safelist_entries' method before calling this method."
            )

        return Query(self.config, self.details_endpoint, Methods.PATCH).set_params(self.payload_params).execute()


    def create_safelist(self):
        """Creates a new safelist library with the corresponding name. 

        You have to call 'set_library_name' before calling this method. 
        
        :returns: HTTP response with safelist library summaries in it's content.
        """
        if not self.payload_params.get("name"):
            raise AttributeError(
                "You must provide a name for the new library. Call the 'set_library_name' method before."
            )

        return Query(self.config, self.summaries_endpoint, Methods.POST).set_params(self.payload_params).execute()


    def delete_entry(self, entry_guid):
        """Deletes an entry from a safelist library. 

        You have to call 'set_library_guid' before calling this method.
        
        :param entry_guid: entry guid to be deleted.
        """
        self._validate_library_guid_is_present()
        endpoint = self.details_endpoint + "/" + entry_guid
        return Query(self.config, endpoint, Methods.DELETE).set_params(self.payload_params).execute()
    

    def delete_safelist(self):
        """Deletes a safelist library. You have to call 'set_library_guid' before
        calling this method."""
        self._validate_library_guid_is_present()
        return Query(self.config, self.details_endpoint, Methods.DELETE).set_params(self.payload_params).execute()


    def extract_terms(self):
        """Extracts IOCs from unstructured text and returns a list of entities ready to be submitted. 

        You have to call 'set_text_to_be_extracted' before calling this method.

        :returns: HTTP response with parsed entities in its content.
        """
        if not self.payload_params.get("text"):
            raise AttributeError(
                "You did not set any text for entities extraction. Call 'set_text_to_be_extracted' before."
            )

        return Query(self.config, self.extract_endpoint, Methods.POST).set_params(self.payload_params).execute()



import pytz
import inspect
import functools
import dateparser

from enum import Enum
from datetime import datetime
from collections import namedtuple



def get_timestamp(date):
    if isinstance(date, int):
        return date
    dt_obj = dateparser.parse(
        date, settings={"TIMEZONE": "UTC", "RETURN_AS_TIMEZONE_AWARE": True}
    )

    timestamp = (dt_obj - datetime(1970, 1, 1, tzinfo=pytz.UTC)).total_seconds() * 1000
    return int(timestamp)


class Methods(Enum):
    POST = 1
    GET = 2
    DELETE = 3
    UPDATE = 4
    PUT = 5
    PATCH = 6


def chained(method):
    """Method decorator to allow chaining."""
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        result = method(self, *args, **kwargs)
        return self if result is None else result
    return wrapper


def fluent(cls):
    """Class decorator to allow method chaining."""
    for name, member in cls.__dict__.items():
        if inspect.isfunction(member) and name.startswith("set_"):
            setattr(cls, name, chained(member))
    return cls


Param = namedtuple('Param', 'key value')


class Params:

    def __init__(self, *args):
        self._dict = {}
        for arg in args:
            self.add(arg)

    def __repr__(self):
        elements = map(repr, self._dict.keys())
        return "%s(%s)" % (self.__class__.__name__, ', '.join(elements))

    def extend(self, args):
        """ Add several items at once. """
        for arg in args:
            self.add(arg)

    def add(self, item):
        """ Add one item to the set. """
        self._dict[item.key] = item

    def get(self, key, default=None):
        try:
            return self._dict[key].value
        except KeyError:
            return default

    def remove(self, item):
        """ Remove an item from the set. """
        del self._dict[item]

    def contains(self, item):
        """ Check whether the set contains a certain item. """
        return item in self._dict.keys()

    # High-performance membership test for Python 2.0 and later
    __contains__ = contains

    def __getitem__(self, index):
        """ Support the 'for item in set:' protocol. """
        return list(self._dict.keys())[index]

    def __iter__(self):
        return iter([value for key, value in self._dict.items()])

    def __len__(self):
        """ Return the number of items in the set """
        return len(self._dict)

    def items(self):
        """ Return a list containing all items in sorted order, if possible (Python 3 only) """
        return self._dict.keys()

    def __copy__(self):
        return Params(self)


class ParamsSerializer(Params):
    def serialize(self):
        return {n.key: n.value for n in self}



from __future__ import unicode_literals
from trustar2.handlers.tags import TagObservable

import pytest

from trustar2.trustar import TruStar
from trustar2 import TagIndicator, TagSubmission

from tests.conftest import BASE_URL

ENCLAVE_ID = "3a93fab3-f87a-407a-9376-8eb3fae99b4e"
IOC_SUBMISSION_GUID = "cc12a5c6-e575-3879-8e41-2bf240cc6fce"
OBSERVABLE_VALUE = "b1fa0ef4930abc0c681081ef2d2f834b3b2fbbbd"
EXPECTED_URL = BASE_URL.format("/{}/cc12a5c6-e575-3879-8e41-2bf240cc6fce/alter-tags")

@pytest.fixture
def tag_indicator(ts):
    return TagIndicator(ts)

@pytest.fixture
def tag_submission(ts):
    return TagSubmission(ts)

@pytest.fixture
def tag_observable(ts):
    return TagObservable(ts)

## Indicator Tags

@pytest.mark.parametrize(
    ("added_tags, removed_tags"),
    (
        (["important", "tag"], []),
        ([], ["not_important", "tag"]),
        (["important", "tag"], ["not_important"])
    )
)
def test_ok_alter_tags_indicators(tag_indicator, mocked_request, added_tags, removed_tags):
    
    mocked_request.post(url=EXPECTED_URL.format("indicators"), json={})
    request = (tag_indicator
               .set_added_tags(added_tags)
               .set_removed_tags(removed_tags)
               .set_enclave_id(ENCLAVE_ID)
               .set_indicator_id(IOC_SUBMISSION_GUID)
               )
    request.alter_tags()

    params = request.payload_params.serialize()
    assert params.get("enclaveGuid") == ENCLAVE_ID
    assert params.get("addedTags") == added_tags
    assert params.get("removedTags") == removed_tags


def test_alter_tag_indicators_incomplete_with_missing_indicator_id(tag_indicator):
    with pytest.raises(AttributeError):
        q = (tag_indicator
             .set_added_tags(["tag"])
             .set_enclave_ids(ENCLAVE_ID)
             .alter_tags())


def test_alter_tag_indicators_incomplete_with_missing_enclave_id(tag_indicator):
    # Missing enclave guid
    with pytest.raises(AttributeError):
        q = (tag_indicator
             .set_added_tags(["tag"])
             .set_indicator_id(IOC_SUBMISSION_GUID)
             .alter_tags())


def test_alter_tag_indicators_incomplete_with_missing_tags(tag_indicator):
    with pytest.raises(AttributeError):
        q = (tag_indicator
             .set_enclave_id(ENCLAVE_ID)
             .set_indicator_id(IOC_SUBMISSION_GUID)
             .alter_tags())

## Submissions Tags

@pytest.mark.parametrize(
    ("added_tags, removed_tags"),
    (
        (["important", "tag"], []),
        ([], ["not_important", "tag"]),
        (["important", "tag"], ["not_important"])
    )
)
def test_ok_alter_tags_submissions(tag_submission, mocked_request, added_tags, removed_tags):
    
    mocked_request.post(url=EXPECTED_URL.format("submissions"), json={})
    request = (tag_submission
               .set_added_tags(added_tags)
               .set_removed_tags(removed_tags)
               .set_enclave_id(ENCLAVE_ID)
               .set_submission_id(IOC_SUBMISSION_GUID)
               )
    request.alter_tags()

    params = request.payload_params.serialize()
    assert params.get("enclaveId") == ENCLAVE_ID
    assert params.get("addedTags") == added_tags
    assert params.get("removedTags") == removed_tags


def test_alter_tag_submissions_incomplete_with_missing_submission_id(tag_submission):
    with pytest.raises(AttributeError):
        q = (tag_submission
             .set_added_tags(["tag"])
             .set_enclave_id(ENCLAVE_ID)
             .alter_tags())


def test_alter_tag_submissions_incomplete_with_missing_tags(tag_submission):
    # Missing added/removed tags
    with pytest.raises(AttributeError):
        q = (tag_submission
             .set_enclave_id(ENCLAVE_ID)
             .set_submission_id(IOC_SUBMISSION_GUID)
             .alter_tags())


def test_alter_tag_submissions_incomplete_with_missing_enclave_id(tag_submission):
    with pytest.raises(AttributeError):
        q = (tag_submission
             .set_added_tags(["tag"])
             .set_submission_id(IOC_SUBMISSION_GUID)
             .alter_tags())


## Observable Tags

@pytest.mark.parametrize(
    ("added_tags, removed_tags"),
    (
        (["important", "tag"], []),
        ([], ["not_important", "tag"]),
        (["important", "tag"], ["not_important"])
    )
)
def test_ok_alter_tags_observables(tag_observable, mocked_request, added_tags, removed_tags):

    EXPECTED_URL = BASE_URL.format("/observables/alter-tags")

    mocked_request.post(url=EXPECTED_URL, json={
        "addedTags": added_tags,
        "removedTags": removed_tags
    })

    request = (tag_observable
               .set_added_tags(added_tags)
               .set_removed_tags(removed_tags)
               .set_enclave_id(ENCLAVE_ID)
               .set_observable_value(OBSERVABLE_VALUE)
               )

    request.alter_tags()

    params = request.payload_params.serialize()
    assert params.get("enclaveGuid") == ENCLAVE_ID
    assert params.get("addedTags") == added_tags
    assert params.get("removedTags") == removed_tags


def test_alter_tag_observables_incomplete_with_missing_observable_value(tag_observable):
    with pytest.raises(AttributeError):
        q = (tag_observable
                .set_added_tags(["tag"])
                .set_enclave_id(ENCLAVE_ID)
                .alter_tags())


def test_alter_tag_observables_incomplete_with_missing_enclave_id(tag_observable):
    with pytest.raises(AttributeError):
        q = (tag_observable
             .set_added_tags(["tag"])
             .set_observable_value(OBSERVABLE_VALUE)
             .alter_tags())


def test_alter_tag_observables_incomplete_with_missing_tags(tag_observable):
    with pytest.raises(AttributeError):
        q = (tag_observable
             .set_enclave_id(ENCLAVE_ID)
             .set_observable_value(IOC_SUBMISSION_GUID)
             .alter_tags())

def test_alter_tag_observables_incomplete_with_empty_tags(tag_observable):
    with pytest.raises(AttributeError):
        q = (tag_observable
                .set_enclave_id(ENCLAVE_ID)
                .set_observable_value(IOC_SUBMISSION_GUID)
                .set_added_tags([])
                .set_removed_tags([])
                .alter_tags())



from __future__ import unicode_literals

import json
import pytest

from trustar2 import Submission
from trustar2.models import Indicator, Entity

from tests.unit.resources import (
    indicators_submission_example_request, 
    non_structured_submission_example_request
)

from tests.conftest import BASE_URL


TOTAL_DEFAULT_PARAMS = 1
BASE_SUBMISSIONS = BASE_URL.format("/submissions{}")
ENCLAVE_ID = "c0f07a9f-76e4-48df-a0d4-c63ed2edccf0"
TIMESTAMP = 1583960400000


@pytest.fixture
def submission(ts):
    return Submission(ts)


@pytest.fixture
def indicators():
    bad_panda = Entity.attribute("MALWARE", "BAD_PANDA")
    related_observable_email = Entity.observable("EMAIL_ADDRESS", "bob@gmail.com")
    related_observable_bad_url = Entity.observable("URL", "badurl.com")
    return [
        Indicator("IP4", "1.2.3.4")
        .set_malicious_score("HIGH")
        .set_attributes(bad_panda)
        .set_related_observables(related_observable_email),
        Indicator("IP4", "5.6.7.8")
        .set_malicious_score("HIGH")
        .set_attributes(bad_panda)
        .set_related_observables(related_observable_bad_url)
        .set_tags(["TAG1"]),
    ]


def test_submission_is_empty(submission):
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS


def test_set_id(submission):
    submission.set_id("TEST_ID")
    params = [p.value for p in submission.payload_params]
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS + 1
    assert "TEST_ID" in params


def test_set_title(submission):
    submission.set_title("TEST_TITLE")
    params = [p.value for p in submission.payload_params]
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS + 1
    assert "TEST_TITLE" in params


def test_set_enclave_id(submission):
    submission.set_enclave_id("TEST-ENCLAVE-ID")
    params = [p.value for p in submission.payload_params]
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS + 1
    assert  "TEST-ENCLAVE-ID" in params


def test_set_external_id(submission):
    submission.set_external_id("TEST-EXTERNAL-ID")
    params = [p.value for p in submission.payload_params]
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS + 1
    assert "TEST-EXTERNAL-ID" in params


def test_set_external_url(submission):
    submission.set_external_url("TEST-EXTERNAL-URL")
    params = [p.value for p in submission.payload_params]
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS + 1
    assert "TEST-EXTERNAL-URL" in params


def test_set_tags(submission):
    submission.set_tags(["TEST_TAG1", "TEST_TAG2"])
    params = [p.value for p in submission.payload_params]
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS
    assert ["TEST_TAG1", "TEST_TAG2"] in params


def test_set_include_content(submission):
    submission.set_include_content(True)
    params = [p.value for p in submission.query_params]
    assert len(submission.query_params) == 1
    assert True in params


def test_set_content_indicators(submission, indicators):
    submission.set_content_indicators(indicators)
    serialized_indicators = [i.serialize() for i in indicators]
    # hacky workaround for python2 unsorted dicts
    params = [p.value for p in submission.payload_params if p.value != []]
    assert serialized_indicators == params[0]["indicators"]


def test_set_raw_content(submission):
    submission.set_raw_content("RAW CONTENT")
    params = [p.value for p in submission.payload_params]
    assert "RAW CONTENT" in params


@pytest.mark.parametrize("date", [TIMESTAMP, "2020-03-11T21:00:00"])
def test_set_timestamp(submission, date):
    submission.set_timestamp(date)
    assert submission.payload_params.get("timestamp") == TIMESTAMP
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS + 1


def test_create_fails_without_mandatory_fields(submission, indicators):
    submission.set_enclave_id("TEST-ENCLAVE_ID")
    submission.set_content_indicators(indicators)
    with pytest.raises(AttributeError):
        submission.create()


@pytest.fixture
def complex_indicator():
    threat_actor = (
        Entity.attribute("THREAT_ACTOR", "ActorName")
        .set_valid_from(1604510497000)
        .set_valid_to(1607102497000)
        .set_confidence_score("LOW")
    )

    malware = (
        Entity.attribute("MALWARE", "MalwareName")
        .set_valid_from(1604510497000)
        .set_valid_to(1607102497000)
        .set_confidence_score("MEDIUM")
    )
    ip4 = (
        Entity.observable("IP4", "2.2.2.2")
        .set_valid_from(1604510497000)
        .set_valid_to(1607102497000)
        .set_confidence_score("LOW")
    )
    url = (
        Entity.observable("URL", "wwww.relatedUrl.com")
        .set_valid_from(1604510497000)
        .set_valid_to(1607102497000)
        .set_confidence_score("HIGH")
    )
    indicator = [
        Indicator("URL", "verybadurl")
        .set_valid_from(1604510497000)
        .set_valid_to(1607102497000)
        .set_confidence_score("LOW")
        .set_malicious_score("BENIGN")
        .set_attributes([threat_actor, malware])
        .set_related_observables([ip4, url])
        .set_properties({"propertyKey": "propertyValue"})
        .set_tags(["importantTag", "anotherTag"])
    ]
    return indicator


@pytest.fixture
def full_submission(submission):
    return (
        submission.set_title("Report, complex test")
        .set_enclave_id(ENCLAVE_ID)
        .set_external_id("external-1234")
        .set_external_url("externalUrlValue")
        .set_timestamp(1607102497000)
        .set_tags(["random_tag"])
    )


@pytest.fixture
def full_iocs_submission(full_submission, complex_indicator):
    return (
        full_submission.set_content_indicators(complex_indicator)
        .set_raw_content("blob of text")
    )


@pytest.fixture
def full_events_submission(full_submission):
    return full_submission.set_content_events("MALICIOUS IP: 8.8.8.8")


@pytest.fixture
def full_intelligence_submission(full_submission):
    return full_submission.set_content_intelligence("MALICIOUS IP: 8.8.8.8")


def test_indicators_submission_ok_json(full_iocs_submission):
    serialized_submission = json.loads(indicators_submission_example_request)
    assert full_iocs_submission.payload_params.serialize() == serialized_submission


def test_non_structured_submission_ok_json(full_events_submission, full_intelligence_submission):
    serialized_subnmission = json.loads(non_structured_submission_example_request)
    assert full_events_submission.payload_params.serialize() == serialized_subnmission
    assert full_intelligence_submission.payload_params.serialize() == serialized_subnmission


def test_indicators_submission_ok(mocked_request, full_iocs_submission):
    expected_url = BASE_SUBMISSIONS.format("/indicators/upsert")
    mocked_request.post(url=expected_url, json={"id": "TEST-ID", "submissionVersion": 1})
    response = full_iocs_submission.upsert()
    assert response.json().get("submissionVersion") == 1
    mocked_request.post(url=expected_url, json={"id": "TEST-ID", "submissionVersion": 2})
    response = full_iocs_submission.upsert()
    assert response.json().get("submissionVersion") == 2


def test_events_submission_ok(mocked_request, full_events_submission):
    expected_url = BASE_SUBMISSIONS.format("/events/upsert")
    mocked_request.post(url=expected_url, json={"id": "TEST-ID", "submissionVersion": 1})
    response = full_events_submission.upsert()
    assert response.json().get("submissionVersion") == 1
    mocked_request.post(url=expected_url, json={"id": "TEST-ID", "submissionVersion": 2})
    response = full_events_submission.upsert()
    assert response.json().get("submissionVersion") == 2


def test_intelligence_submission_ok(mocked_request, full_intelligence_submission):
    expected_url = BASE_SUBMISSIONS.format("/intelligence/upsert")
    mocked_request.post(url=expected_url, json={"id": "TEST-ID", "submissionVersion": 1})
    response = full_intelligence_submission.upsert()
    assert response.json().get("submissionVersion") == 1
    mocked_request.post(url=expected_url, json={"id": "TEST-ID", "submissionVersion": 2})
    response = full_intelligence_submission.upsert()
    assert response.json().get("submissionVersion") == 2


def test_changing_content_will_change_url(full_submission):
    full_submission.set_content_indicators([Indicator("IP4", "8.8.8.8")])
    assert full_submission._submission_category == "/indicators"
    
    full_submission.set_content_events("test-event")
    assert full_submission._submission_category == "/events"
    
    full_submission.set_content_intelligence("test-intelligence")
    assert full_submission._submission_category == "/intelligence"


def test_get_structured_indicators_submissions(submission, mocked_request):
    json_response = json.loads(indicators_submission_example_request)
    endpoint = "/indicators?id=external-1234&idType=EXTERNAL&enclaveGuid={}&includeContent=true".format(ENCLAVE_ID)
    expected_url = BASE_SUBMISSIONS.format(endpoint)
    mocked_request.get(expected_url, json=json_response)

    submission.set_enclave_id(ENCLAVE_ID)
    submission.set_external_id("external-1234")
    submission.set_id_type_as_external(True)
    submission.set_include_content(True)

    response = submission.get(structured_indicators=True)
    assert response.json() == json_response


def test_get_non_structured_submissions(submission, mocked_request):
    json_response = json.loads(non_structured_submission_example_request)
    endpoint = "/events?id=external-1234&idType=EXTERNAL&enclaveGuid={}&includeContent=true".format(ENCLAVE_ID)
    expected_url = BASE_SUBMISSIONS.format(endpoint)
    mocked_request.get(expected_url, json=json_response)

    submission.set_enclave_id(ENCLAVE_ID)
    submission.set_external_id("external-1234")
    submission.set_id_type_as_external(True)
    submission.set_include_content(True)

    response = submission.get(structured_indicators=False)
    assert response.json() == json_response


def test_get_submission_status(submission, mocked_request):
    json_response = {"id": "test-submission-id", "status": "SUBMISSION_SUCCESS"}
    expected_url = BASE_SUBMISSIONS.format("/test-submission-id/status")
    mocked_request.get(expected_url, json=json_response)
    response = submission.get_submission_status("test-submission-id")
    assert response.json() == json_response

def test_set_sort_order_not_implemented(submission):
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS
    with pytest.raises(NotImplementedError):
        submission.set_sort_order("ASC")
    assert len(submission.payload_params) == TOTAL_DEFAULT_PARAMS



from __future__ import unicode_literals

from trustar2.query import Query
from trustar2.trustar_enums import MaxValues
from trustar2.handlers.tags import TagSubmission
from trustar2.base import fluent, Methods, get_timestamp
from trustar2.handlers.search_handler import SearchHandler


@fluent
class Submission(SearchHandler):

    SUBMISSION_MANDATORY_FIELDS = ("title", "content", "enclaveGuid")
    _path = "/submissions"
    _submission_category = "/indicators"

    def __init__(self, config=None):
        super(Submission, self).__init__(config)
        self.set_tags()

    def __str__(self):
        return "Submission <{}> with external Id <{}>".format(
            self.payload_params.get("title"), 
            self.payload_params.get("externalId")
        )

    @property
    def endpoint(self):
        return self.config.request_details.get("api_endpoint") + self._path


    def set_id(self, submission_id):
        """Adds id param to set of params.

        :param submission_id: field value.
        :returns: self.
        """
        self.set_payload_param("id", submission_id)
        self.set_query_param("id", submission_id)


    def set_title(self, title):
        """Adds title param to set of params.

        :param title: field value.
        :returns: self.
        """
        self.set_payload_param("title", title)


    def set_content_indicators(self, indicators):
        """Adds content param to set of params.

        :param indicators: field value. List of Indicator objects.
        :returns: self.
        """
        self._submission_category = "/indicators"
        indicators = [i.serialize() for i in indicators]
        if len(indicators) > MaxValues.INDICATORS.value:
            indicators = indicators[:MaxValues.INDICATORS.value]

        content = {"indicators": indicators}
        self.set_payload_param("content", content)


    def set_content_events(self, content):
        """Adds content param to set of params.

        :param indicators: field value. List of Indicator objects.
        :returns: self.
        """
        self._submission_category = "/events"
        self.set_payload_param("content", content)


    def set_content_intelligence(self, content):
        """Adds content param to set of params.

        :param indicators: field value. List of Indicator objects.
        :returns: self.
        """
        self._submission_category = "/intelligence"
        self.set_payload_param("content", content)


    def set_enclave_id(self, enclave_id):
        """Adds enclaveId param to set of params.

        :param enclave_id: field value.
        :returns: self.
        """
        self.set_payload_param("enclaveGuid", enclave_id)
        self.set_query_param("enclaveGuid", enclave_id)


    def set_external_id(self, external_id):
        """Adds externalId param to set of params.

        :param external_id: field value.
        :returns: self.
        """
        self.set_payload_param("externalId", external_id)


    def set_external_url(self, external_url):
        """Adds externalUrl param to set of params.

        :param external_url: field value.
        :returns: self.
        """
        self.set_payload_param("externalUrl", external_url)


    def set_tags(self, tags=None):
        """Adds tags param to set of params.

        :param tags: field value.
        :returns: self.
        """
        if not tags:
            tags = []

        if len(tags) > MaxValues.TAGS.value:
            tags = tags[:MaxValues.TAGS.value]

        self.set_payload_param("tags", tags)


    def set_id_type_as_external(self, external=False):
        """Sets idType to EXTERNAL if 'external' parameter is True.
        
        :param external: boolean indicating if id is external or not
        :returns: self.
        """
        if external:
            self.set_query_param("idType", "EXTERNAL")


    def set_include_content(self, content=False):
        """
        Adds includeContent param to set of params.

        :param content: field value.
        :returns: self.
        """
        self.set_query_param("includeContent", content)


    def set_timestamp(self, timestamp):
        """Adds timestamp param to set of params.

        :param timestamp: field value.
        :returns: self.
        """
        if not isinstance(timestamp, int):
            timestamp = get_timestamp(timestamp)

        self.set_payload_param("timestamp", timestamp)


    def set_raw_content(self, raw_content):
        """Adds rawContent param to set of params.

        :param raw_content: field value.
        :returns: self.
        """
        self.set_payload_param("rawContent", raw_content)


    def set_submission_version(self, version):
        """Adds submissionVersion param to set of params.

        :param version: field value.
        :returns: self.
        """
        self.set_payload_param("submissionVersion", version)


    @property
    def query_string_params(self):
        query_params = self.query_params.serialize()
        if self.should_use_external_id():
            query_params["id"] = self.payload_params.get("externalId")

        return query_params


    def should_use_external_id(self):
        """Returns True if params are set to retrieve a submission by external id"""
        return "idType" in self.query_params and "externalId" in self.payload_params


    def create_query(self, method, specific_endpoint=""):
        """Returns a new instance of a Query object according config, endpoint and method."""
        endpoint = self.endpoint + specific_endpoint
        return Query(self.config, endpoint, method)


    def _raise_without_id(self):
        if not "id" in self.query_string_params:
            raise AttributeError(
                "You need to set an id, or an external id marking the id type as external"
            )


    def delete(self):
        """Deletes a submission according to query_params set before."""
        self._raise_without_id()
        return (
            self.create_query(Methods.DELETE, specific_endpoint=self._submission_category)
            .set_query_string(self.query_string_params)
            .execute()
        )


    def get(self, structured_indicators=True):
        """Retrieves a submission according to query_params set before."""
        self._submission_category = "/events" if not structured_indicators else "/indicators"
        self._raise_without_id()
        return (
            self.create_query(Methods.GET, specific_endpoint=self._submission_category)
            .set_query_string(self.query_string_params)
            .execute()
        )


    def upsert(self):
        """Update a submission if it already exists or create a new one if it doesn't."""
        for k in self.SUBMISSION_MANDATORY_FIELDS:
            if k not in self.payload_params:
                raise AttributeError("{} field should be in your submission".format(k))

        return (
            self.create_query(Methods.POST, specific_endpoint=self._submission_category + "/upsert")
            .set_params(self.payload_params)
            .set_query_string(self.query_string_params)
            .execute()
        )


    def search(self):
        """Search for submissions (intel, events and indicators)"""
        self._validate_dates()
        endpoint = "/search"
        return (
            self.create_query(Methods.POST, specific_endpoint=endpoint)
            .set_params(self.payload_params)
            .set_query_string({"pageSize": self.query_params.get("pageSize", 25)})
        )


    def get_submission_status(self, submission_id):
        """Returns submission status for a given submission id"""
        endpoint = "/{}/status".format(submission_id)
        return self.create_query(Methods.GET, specific_endpoint=endpoint).execute()


    def tags(self):
        """Returns a TagSubmission handler that will let you alter-tags on a submission"""
        return TagSubmission(self.config)

    def set_sort_order(self, order):
        raise NotImplementedError("Submissions dont support sort order")



from __future__ import unicode_literals

from trustar2.query import Query
from trustar2.models import Entity
from trustar2.base import fluent, Methods
from trustar2.handlers.tags import TagIndicator
from trustar2.handlers.search_handler import SearchHandler
from trustar2.trustar_enums import ObservableTypes, SortOrder, AttributeTypes

 

@fluent
class SearchIndicator(SearchHandler):
    url = "/indicators"

    def __init__(self, config=None):
        super(SearchIndicator, self).__init__(config)
        self.endpoint = None


    @property
    def base_url(self):
        return self.config.request_details.get("api_endpoint") + self.url


    @staticmethod
    def _get_entity(entity, entity_type):
        if isinstance(entity, Entity):
            return entity.params.serialize()[entity.key]

        if isinstance(entity, dict):
            entity_type = (
                AttributeTypes
                if issubclass(entity_type, AttributeTypes)
                else ObservableTypes
            )
            return {
                "type": SearchIndicator._get_value(entity.get("type"), entity_type),
                "value": entity.get("value"),
            }

        raise AttributeError(
            "List elements should be a dictionary or the entity class"
        )


    def set_priority_scores(self, scores):
        if not isinstance(scores, list) or any([s for s in scores if s > 3 or s < -1]):
            raise AttributeError("scores should be a list of integers between -1 and 3")
        self.set_payload_param("priorityScores", scores)


    def set_observable_types(self, types):
        if not isinstance(types, list):
            raise AttributeError("types should be a list")

        types = [self._get_value(t, ObservableTypes) for t in types]
        self.set_payload_param("types", types)


    def set_attributes(self, attributes):
        if not isinstance(attributes, list):
            raise AttributeError("attributes should be a list")

        attributes = [self._get_entity(e, AttributeTypes) for e in attributes]
        self.set_payload_param("attributes", attributes)


    def set_related_observables(self, observables):
        if not isinstance(observables, list):
            raise AttributeError("observables should be a list")

        observables = [self._get_entity(o, ObservableTypes) for o in observables]
        self.set_payload_param("relatedObservables", observables)


    def set_include_safelisted(self, include_safelisted):
        self.set_payload_param("includeSafelisted", include_safelisted)


    def create_query(self, method):
        return Query(self.config, self.endpoint, method)


    def search(self):
        self._validate_dates()
        self.endpoint = self.base_url + "/search"
        return (
            self.create_query(Methods.POST)
            .set_params(self.payload_params)
            .set_query_string({"pageSize": self.query_params.get("pageSize", 25)})
        )


    def tags(self):
        return TagIndicator(self.config)



from __future__ import unicode_literals

from .base import Base
from .entity import Entity
from trustar2.base import fluent
from trustar2.trustar_enums import ObservableTypes, MaxValues


@fluent
class Indicator(Base):
    def __init__(self, entity_type, value):
        self.observable = Entity(ObservableTypes, entity_type, value, alias="observable")
        self.attributes = []
        self.related_observables = []
        self.tags = []

    def set_related_observables(self, related_obs):
        if isinstance(related_obs, list):
            self.related_observables += related_obs
        else:
            self.related_observables.append(related_obs)

        if len(self.related_observables) > MaxValues.RELATED_OBSERVABLES.value:
            self.related_observables = self.related_observables[:MaxValues.RELATED_OBSERVABLES.value]

    def set_attributes(self, related_attribute):
        if isinstance(related_attribute, list):
            self.attributes += related_attribute
        else:
            self.attributes.append(related_attribute)

        if len(self.attributes) > MaxValues.ATTRIBUTES.value:
            self.attributes = self.attributes[:MaxValues.ATTRIBUTES.value]

    def set_valid_to(self, valid_to):
        self.observable.set_valid_to(valid_to)

    def set_valid_from(self, valid_from):
        self.observable.set_valid_from(valid_from)

    def set_malicious_score(self, mal_score):
        self.observable.set_malicious_score(mal_score)

    def set_confidence_score(self, conf_score):
        self.observable.set_confidence_score(conf_score)

    def set_properties(self, properties):
        self.observable.set_properties(properties)

    def set_tags(self, tag):
        if isinstance(tag, list):
            self.tags += tag
        else:
            self.tags.append(tag)

        if len(self.tags) > MaxValues.TAGS.value:
            self.tags = self.tags[:MaxValues.TAGS.value]

    def serialize(self):
        serialized = {}
        serialized.update(self.observable.serialize())
        serialized.update({"attributes": [attr.serialize() for attr in self.attributes] if len(self.attributes) else []})
        serialized.update({"relatedObservables": [attr.serialize() for attr in self.related_observables] if len(self.related_observables) else []})
        serialized.update({"tags": self.tags})
        return serialized



from __future__ import unicode_literals

from trustar2.query import Query
from trustar2.base import fluent, Methods
from trustar2.handlers.tags import TagObservable
from trustar2.handlers.search_handler import SearchHandler
from trustar2.trustar_enums import ID_Types, ObservableSortColumns, ObservableTypes, MaxValues

MAX_TAGS = MaxValues.TAGS.value 

@fluent
class Observables(SearchHandler):

    _base_endpoint = "/observables"
    
    _get_from_submission_endpoint = ""
    _search_endpoint = "/search"

    def __init__(self, config=None):
        super(Observables, self).__init__(config)

    @property
    def base_url(self):
        return self.config.request_details.get("api_endpoint") + self._base_endpoint

    def search(self):
        """
        Prepare the query to search the observables with the specified filters.

        :returns: Query Object.
        """
        self._validate_search_params()
        url = "{}{}".format(self.base_url, self._search_endpoint)
        query = Query(self.config, url, Methods.POST, query_string=self.query_params.serialize())
        return query.set_params(self.payload_params)
    
    def get_from_submission(self, submission_id, id_type = ID_Types.INTERNAL.value):
        """
        Prepare the query to get the observables from a submission.

        :param submission_id: The guid of the submission
        :param id_type (optional): 'INTERNAL' | 'EXTERNAL' | 'UNRECOGNIZED'
        :returns: Query Object.
        """
        url = "{}{}".format(self.base_url, self._get_from_submission_endpoint)
        self.set_query_param("submissionId", submission_id)
        self.set_query_param("idType", id_type)
        query = Query(self.config, url, Methods.GET)
        return query.set_query_string(self.query_params)

    def set_search_types(self, types):
        """
        Adds types to the search filters.

        :param types: Array of ObservableTypes Values
        :returns: self.
        """
        types = self._argument_to_unique_list(types)
        types = list(map(lambda t: self._get_value(t, ObservableTypes), types))
        self.set_payload_param("types", types)
    
    def set_sort_column(self, column):
        """
        Specify the column to sort the search results.

        :param column: 'FIRST_SEEN' | 'LAST_SEEN'
        :returns: self.
        """
        return super(Observables, self).set_sort_column(column, ObservableSortColumns)

    def tags(self):
        """
        Returns an observable tags handler

        :returns: TagObservable handler.
        """
        return TagObservable(self.config)

    def _validate_tags_length(self):
        included_tags = self.payload_params.get("includedTags", [])
        excluded_tags = self.payload_params.get("excludedTags", [])
        are_too_may_tags = (len(included_tags) > MAX_TAGS) or (len(excluded_tags) > MAX_TAGS)
        if are_too_may_tags:
            raise AttributeError("Tags are limited to {} per observable".format(MAX_TAGS))

    def _validate_search_params(self):
        self._validate_dates()
        self._validate_tags_length()



from enum import Enum


class TSEnum(Enum):

    @classmethod
    def members(cls):
        return (m.value for m in cls)


class SortColumns(TSEnum):
    
    CREATED = "CREATED"
    PROCESSED_AT = "PROCESSED_AT"
    UPDATED = "UPDATED"


class SortOrder(TSEnum):
    ASC = "ASC"
    DESC = "DESC"


class ObservableTypes(TSEnum):

    BITCOIN_ADDRESS = "BITCOIN_ADDRESS"
    CIDR_BLOCK = "CIDR_BLOCK"
    EMAIL_ADDRESS = "EMAIL_ADDRESS"
    IP4 = "IP4"
    IP6 = "IP6"
    MD5 = "MD5"
    PHONE_NUMBER = "PHONE_NUMBER"
    REGISTRY_KEY = "REGISTRY_KEY"
    SHA1 = "SHA1"
    SHA256 = "SHA256"
    SOFTWARE = "SOFTWARE"
    URL = "URL"
    X_ID = "X_ID"
    DOMAIN = 'DOMAIN'


class AttributeTypes(TSEnum):
    
    CORA_MALWARE = "CORA_MALWARE"
    CVE = "CVE"
    MALWARE = "MALWARE"
    MITRE_TACTIC = "MITRE_TACTIC"
    THREAT_ACTOR = "THREAT_ACTOR"


class TruStarUrls(TSEnum):

    API = "https://api.trustar.co/api/2.0"
    AUTH_TOKEN = "https://api.trustar.co/oauth/token"
    STATION = "https://station.trustar.co"


class MaxValues(TSEnum):

    TAGS = 20
    RELATED_OBSERVABLES = 50
    ATTRIBUTES = 50
    INDICATORS = 2000


class WorkflowDestinations(TSEnum):

    QRADAR = "QRADAR"
    ENCLAVE = "ENCLAVE"
    UNKNOWN = "UNKNOWN"

class ID_Types(TSEnum):
    INTERNAL = "INTERNAL"
    EXTERNAL = "EXTERNAL"
    UNRECOGNIZED = "UNRECOGNIZED"

class ObservableSortColumns(TSEnum):
    FIRST_SEEN = "FIRST_SEEN"
    LAST_SEEN = "LAST_SEEN"



from __future__ import unicode_literals
import json
from .resources import observables_search_example_request
from trustar2.handlers.observables import Observables
from trustar2.handlers.tags import TagObservable
from trustar2.trustar_enums import MaxValues
import pytest


TEST_DATE = [1583960400000, "2020-03-11T21:00:00"]
MAX_TAGS = MaxValues.TAGS.value

@pytest.fixture
def observables(ts):
    return Observables(ts)

def test_set_search_types_valid(observables):
    assert len(observables.payload_params) == 0
    types = ["MD5", "SOFTWARE"]
    observables.set_search_types(types)
    assert len(observables.payload_params) == 1
    assert sorted(observables.payload_params.get("types")) == types

def test_set_search_types_invalid(observables):
    assert len(observables.payload_params) == 0
    types = ["INVALID_TYPE", "MD5"]
    with pytest.raises(AttributeError):
        observables.set_search_types(types)
    assert len(observables.payload_params) == 0

def test_set_sort_column_valid(observables):
    assert len(observables.payload_params) == 0
    observables.set_sort_column("FIRST_SEEN")
    assert observables.payload_params.get("sortColumn") == "FIRST_SEEN"
    assert len(observables.payload_params) == 1
    observables.set_sort_column("LAST_SEEN")
    assert observables.payload_params.get("sortColumn") == "LAST_SEEN"
    assert len(observables.payload_params) == 1

def test_set_sort_column_vinalid(observables):
    assert len(observables.payload_params) == 0
    with pytest.raises(AttributeError):
        observables.set_sort_column("UPDATED")
    assert len(observables.payload_params) == 0

def tags(observables):
    tags = observables.tags()
    assert(isinstance(tags, TagObservable))

def test_search_too_many_included_tags(observables):
    tags = list(range(0, MAX_TAGS + 1))
    observables.set_included_tags(tags)
    with pytest.raises(AttributeError):
        observables.search()

def test_search_too_many_excluded_tags(observables):
    tags = list(range(0, MAX_TAGS + 1))
    observables.set_excluded_tags(tags)
    with pytest.raises(AttributeError):
        observables.search()


def test_search_invalid_dates(observables):
    observables.set_from("1 day ago")
    observables.set_to("2 days ago")
    with pytest.raises(AttributeError):
        observables.search()

def test_search(observables):
    types = ["MD5"]
    query_term = "query"
    enclave_ids = ["4bdc3f5b-3ed5-4d99-b20c-2d801866ef0b"]
    from_date = 1596607968000
    to_date = 1598308171000

    q = (
        observables.set_query_term(query_term)
        .set_enclave_ids(enclave_ids)
        .set_search_types(types)
        .set_from(from_date)
        .set_to(to_date)
        .set_sort_column("FIRST_SEEN")
        .set_sort_order("ASC")
        .search()
    )
    assert q.params.serialize() == json.loads(observables_search_example_request)

def test_get_from_submission(observables):
    submission_id = "2d87abca-3b6c-4bdc-b9dd-0a3e29901bbb"
    q = observables.get_from_submission(submission_id)
    assert(q.query_string.get("submissionId") ==  submission_id)



from __future__ import unicode_literals

from datetime import datetime
from trustar2.base import fluent, get_timestamp
from trustar2.trustar_enums import  SortColumns,  SortColumns, SortOrder
from trustar2.handlers.base_handler import BaseHandler


@fluent
class SearchHandler(BaseHandler):

    def __init__(self, config=None):
        super(SearchHandler, self).__init__(config)


    @staticmethod
    def _get_value(arg, enum):
        if isinstance(arg, enum):
            return arg.value

        if isinstance(arg, str) and arg in enum.members():
            return arg

        if isinstance(arg, type("")) and arg in enum.members():
            return arg  # For py2 and py3 compatibility

        raise AttributeError(
            "Possible value types are: {}".format(list(enum.members()))
        )

    def _validate_dates(self):
        from_date = self.payload_params.get("from")
        to_date = self.payload_params.get("to")
        if from_date and to_date:
            from_date_dt = datetime.fromtimestamp(from_date / 1000)
            to_date_dt = datetime.fromtimestamp(to_date / 1000)
            if (to_date_dt - from_date_dt).days > 364:
                raise AttributeError("Time window can not be greater than 1 year.")

            if (from_date_dt > to_date_dt):
                raise AttributeError("'from' can not be a date after 'to'.")

        if from_date and not to_date:
            from_date_dt = datetime.fromtimestamp(from_date / 1000)
            if (datetime.today() - from_date_dt).days > 364:
                raise AttributeError("Time window can not be greater than 1 year.")


    def set_query_term(self, query):
        self.set_payload_param("queryTerm", query)


    def set_from(self, from_date):
        self.set_payload_param("from", get_timestamp(from_date))


    def set_to(self, to_date):
        self.set_payload_param("to", get_timestamp(to_date))


    def set_enclave_ids(self, enclave_guids):
        self.set_payload_param("enclaveGuids", self._argument_to_unique_list(enclave_guids))


    def set_included_tags(self, tags):
        self.set_payload_param("includedTags", self._argument_to_unique_list(tags))


    def set_excluded_tags(self, tags):
        self.set_payload_param("excludedTags", self._argument_to_unique_list(tags))
        

    def set_sort_column(self, column, options = SortColumns):
        column = self._get_value(column, options)
        self.set_payload_param("sortColumn", column)


    def set_sort_order(self, order):
        order = self._get_value(order, SortOrder)
        self.set_payload_param("sortOrder", order)


    def set_page_size(self, page_size):
        self.set_query_param("pageSize", page_size)



# Seems like from time to time people do a from trustar2 import *, lets minimize the chances of something ugly happening
__all__ = ["get_logger", "TrustarJSONFormatter"]

import datetime
import logging
import os
import sys

import json_log_formatter

from .config import LOGGING_ENV_VAR


class TrustarJSONFormatter(json_log_formatter.JSONFormatter):
    """
    Custom class to override the default behaviour of the JSONFormatter
    """

    def format(self, record):
        """
        the default behaviour of JSONFormatter is to cast everything into a string
        we avoid calling getMessage and leave the object as it is, this way our json messages can have nested dicts
        """
        message = record.msg
        extra = self.extra_from_record(record)
        json_record = self.json_record(message, extra, record)
        mutated_record = self.mutate_json_record(json_record)
        # Backwards compatibility: Functions that overwrite this but don't
        # return a new value will return None because they modified the
        # argument passed in.
        if mutated_record is None:
            mutated_record = json_record
        return self.to_json(mutated_record)

    def json_record(self, message, extra, record):
        extra['message'] = message
        extra['level'] = record.levelname
        extra['module'] = record.name
        extra['time'] = datetime.datetime.utcnow()
        if record.exc_info:
            extra['exec_info'] = self.formatException(record.exc_info)
        return extra

    def to_json(self, record):
        """Converts record dict to a JSON string.

        It makes best effort to serialize a record (represents an object as a string)
        instead of raising TypeError if json library supports default argument.
        Note, ujson doesn't support it.
        """
        return self.json_lib.dumps(record, default=_json_object_encoder)


def _json_object_encoder(obj):
    try:
        return obj.to_json()
    except AttributeError:
        return str(obj)


def get_handler():
    """
    Gets the handler to manage the output of the logger, default: stdout
    """
    handler = logging.StreamHandler(sys.stdout)
    # TODO: read from a config file, or env var, and override the default formatter.
    # IE: logging.FileHandler(filename='/path/to/file.log')
    return handler


def get_formatter():
    formatter = TrustarJSONFormatter
    # TODO: read from a config file, or env var, and override the default formatter.
    return formatter()


def get_logging_level():
    return int(os.environ.get(LOGGING_ENV_VAR, logging.INFO))


output_handler = get_handler()
output_handler.setFormatter(get_formatter())


def get_logger(name=None):
    logger = logging.getLogger(name or __name__)
    logger.addHandler(output_handler)
    logger.setLevel(get_logging_level())
    return logger



import json
import pytest

from trustar2 import Workflow, TruStar
from trustar2.models import WorkflowConfig
from tests.unit.resources import serialized_workflow_config

URL = "https://api.trustar.co/api/2.0/workflows"
TIMESTAMP = 1583960400000
DEFAULT_PARAMS = 1

@pytest.fixture
def workflow(ts):
    return Workflow(ts)


@pytest.fixture
def workflow_config():
    wf_config = WorkflowConfig()
    wf_config.set_observable_types(["URL", "IP4", "IP6", "SHA256"])
    wf_config.set_priority_scores(["MEDIUM", "HIGH"])
    wf_config.set_source_configs([
        ("test-enclave-id", 3),
        ("test-enclave-id2", 3), 
        ("test-enclave-id3", 1),
        ("test-enclave-id4", 5)
    ])
    wf_config.set_destination_configs(("test-enclave-id", "ENCLAVE"))
    return wf_config


@pytest.fixture
def serialized_wf_config():
    return json.loads(serialized_workflow_config)


@pytest.fixture
def wf_response(serialized_wf_config):
    return {
        "guid": "test-workflow-id",
        "name": "Test Workflow Name",
        "created": TIMESTAMP,
        "updated": TIMESTAMP,
        "workflowConfig": serialized_wf_config
    }


def test_workflow_is_initially_empty(workflow):
    assert len(workflow.payload_params) == 0 + DEFAULT_PARAMS
    assert len(workflow.query_params) == 0


def test_set_type(workflow):
    workflow.set_type("INDICATOR_PRIORITIZATION")
    assert len(workflow.query_params) == 1
    assert workflow.query_params.get("type") == "INDICATOR_PRIORITIZATION"


def test_set_name(workflow):
    workflow.set_name("TEST-WORKFLOW-NAME")
    assert len(workflow.query_params) == 1
    assert len(workflow.payload_params) == 1 + DEFAULT_PARAMS
    assert workflow.query_params.get("name") == "TEST-WORKFLOW-NAME"
    assert workflow.payload_params.get("name") == "TEST-WORKFLOW-NAME" 


def test_set_name_with_string_outside_boundaries(workflow):
    with pytest.raises(AttributeError):
        workflow.set_name("NO")

    with pytest.raises(AttributeError):
        workflow.set_name("N" * 121)


@pytest.mark.parametrize("date", [TIMESTAMP, "2020-03-11T21:00:00"])
def test_set_created_from(workflow, date):
    workflow.set_created_from(date)
    assert len(workflow.query_params) == 1
    assert workflow.query_params.get("createdFrom") == TIMESTAMP


@pytest.mark.parametrize("date", [TIMESTAMP, "2020-03-11T21:00:00"])
def test_set_created_to(workflow, date):
    workflow.set_created_to(date)
    assert len(workflow.query_params) == 1
    assert workflow.query_params.get("createdTo") == TIMESTAMP


@pytest.mark.parametrize("date", [TIMESTAMP, "2020-03-11T21:00:00"])
def test_set_updated_from(workflow, date):
    workflow.set_updated_from(date)
    assert len(workflow.query_params) == 1
    assert workflow.query_params.get("updatedFrom") == TIMESTAMP


@pytest.mark.parametrize("date", [TIMESTAMP, "2020-03-11T21:00:00"])
def test_set_updated_to(workflow, date):
    workflow.set_updated_to(date)
    assert len(workflow.query_params) == 1
    assert workflow.query_params.get("updatedTo") == TIMESTAMP


def test_set_workflow_id(workflow):
    workflow.set_workflow_id("test-workflow-id")
    assert workflow.workflow_guid == "test-workflow-id"


def test_set_safelist_ids(workflow):
    workflow.set_safelist_ids(["test-safelist-id1", "test-safelist-id2"])
    assert len(workflow.payload_params) == 1
    assert workflow.payload_params.get("safelistGuids") == ["test-safelist-id1", "test-safelist-id2"]


def test_set_workflow_config(workflow, workflow_config):
    workflow.set_workflow_config(workflow_config)
    assert len(workflow.payload_params) == 1 + DEFAULT_PARAMS
    assert workflow.payload_params.get("workflowConfig") == json.loads(serialized_workflow_config)


def test_workflow_creation_successful(workflow, mocked_request, workflow_config, wf_response):
    mocked_request.post(URL, json=wf_response)
    workflow.set_name("Test Workflow Name")
    workflow.set_workflow_config(workflow_config)
    workflow.set_safelist_ids([])
    response = workflow.create()
    assert response.status_code == 200
    assert response.json() == wf_response


def test_workflow_creation_fails_with_missing_payload_field(workflow, workflow_config):
    with pytest.raises(AttributeError):
        workflow.set_workflow_config(workflow_config)
        workflow.set_safelist_ids([])
        response = workflow.create()


def test_workflow_get_by_id(workflow, mocked_request, wf_response):
    mocked_request.get(URL + "/test-workflow-id", json=wf_response)
    workflow.set_workflow_id("test-workflow-id")
    response = workflow.get_by_id()
    assert response.status_code == 200
    assert response.json() == wf_response


def test_workflow_get_by_id_fails_with_missing_id(workflow, wf_response):
    with pytest.raises(AttributeError):
        response = workflow.get_by_id()


def test_workflow_get_all_workflows(workflow, mocked_request, wf_response):
    mocked_request.get(URL, json=[wf_response])
    response = workflow.get()
    assert response.status_code == 200
    assert response.json() == [wf_response]


def test_delete_workflow_by_id(workflow, mocked_request):
    mocked_request.delete(URL + "/test-workflow-id", status_code=200)
    workflow.set_workflow_id("test-workflow-id")
    response = workflow.delete()
    assert response.status_code == 200


def test_workflow_delete_fails_with_missing_id(workflow, wf_response):
    with pytest.raises(AttributeError):
        response = workflow.delete()


def test_workflow_update_successful(workflow, mocked_request, workflow_config, wf_response):
    mocked_request.put(URL + "/test-workflow-id", json=wf_response)
    workflow.set_name("Test Workflow Name")
    workflow.set_workflow_config(workflow_config)
    workflow.set_safelist_ids([])
    workflow.set_workflow_id("test-workflow-id")
    response = workflow.update()
    assert response.status_code == 200
    assert response.json() == wf_response


def test_workflow_update_fails_with_missing_id(workflow, workflow_config):
    with pytest.raises(AttributeError):
        workflow.set_name("Test Workflow Name")
        workflow.set_workflow_config(workflow_config)
        workflow.set_safelist_ids([])
        response = workflow.update()



import pytest

from trustar2.trustar import TruStar

proxy = {"https": "https://user:pass@le.proxy.com", "http":None}


@pytest.fixture
def trustar_with_proxy():
    return TruStar(api_key="xxxx", api_secret="xxx", client_metatag="test_env", proxy=proxy)


def test_trustar_proxy(ts):
    assert ts.get_proxy() == {}


def test_trustar_with_proxy(trustar_with_proxy):
    assert trustar_with_proxy.get_proxy() == {"https": "https://user:pass@le.proxy.com"}



from trustar2.query import Query
from trustar2.handlers.base_handler import BaseHandler
from trustar2.base import fluent, Methods, get_timestamp


MIN_NAME_LEN = 3
MAX_NAME_LEN = 120


@fluent
class Workflow(BaseHandler):

    _path = "/workflows"

    def __init__(self, config=None):
        super(Workflow, self).__init__(config)
        self.workflow_guid = None
        self.set_safelist_ids()


    @property
    def endpoint(self):
        return self.config.request_details.get("api_endpoint") + self._path


    def set_type(self, type):
        self.set_query_param("type", type)


    def set_name(self, name):
        if len(name) < MIN_NAME_LEN or len(name) > MAX_NAME_LEN:
            raise AttributeError("Workflow name's length must be between {} and {} characters".format(
                MIN_NAME_LEN, MAX_NAME_LEN
            ))

        self.set_query_param("name", name)
        self.set_payload_param("name", name)


    def set_created_from(self, created_from):
        if not isinstance(created_from, int):
            created_from = get_timestamp(created_from)

        self.set_query_param("createdFrom", created_from)


    def set_created_to(self, created_to):
        if not isinstance(created_to, int):
            created_to = get_timestamp(created_to)

        self.set_query_param("createdTo", created_to)

    
    def set_updated_from(self, updated_from):
        if not isinstance(updated_from, int):
            updated_from = get_timestamp(updated_from)

        self.set_query_param("updatedFrom", updated_from)

    
    def set_updated_to(self, updated_to):
        if not isinstance(updated_to, int):
            updated_to = get_timestamp(updated_to)

        self.set_query_param("updatedTo", updated_to)


    def set_workflow_id(self, workflow_id):
        if not isinstance(workflow_id, str):
            raise AttributeError("Workflow ID must be a string.")

        self.workflow_guid = workflow_id


    def set_workflow_config(self, workflow_config):
        self.set_payload_param("workflowConfig", workflow_config.serialize())

    
    def set_safelist_ids(self, safelist_ids=[]):
        if not isinstance(safelist_ids, list):
            raise AttributeError("'safelist_ids' must be a list of safelist_guids (strings).")

        self.set_payload_param("safelistGuids", safelist_ids)


    def create_query(self, method, specific_endpoint=""):
        """Returns a new instance of a Query object according config, endpoint and method."""
        endpoint = self.endpoint + specific_endpoint
        return Query(self.config, endpoint, method)


    def _raise_if_payload_is_not_set_up(self):
        name = self.payload_params.get("name")
        workflow_config = self.payload_params.get("workflowConfig")
        safelist = self.payload_params.get("safelistGuids")
        if name is None or workflow_config is None or safelist is None:
            raise AttributeError("You have to set the name, workflow_config and safelist_ids for the workflow.")


    def _raise_if_workflow_id_is_not_set_up(self):
        if self.workflow_guid is None:
            raise AttributeError("You have to set up the workflow id.")

    
    def create(self):
        """
        Creates a new workflow in TruSTAR platform.
        
        You'll need to call 'set_name' and 'set_workflow_config' before 
        calling to this method.
        """
        self._raise_if_payload_is_not_set_up()
        return self.create_query(Methods.POST).set_params(self.payload_params).execute()


    def get(self):
        """
        Gets all workflows in TruSTAR platform.
        You can optionally call any of the following methods to filter the results:
            - set_type
            - set_name
            - set_created_from
            - set_created_to
            _ set_updated_from
            - set_updated_to
        """
        return (
            self.create_query(Methods.GET)
            .set_query_string(self.query_params.serialize())
            .set_params(self.payload_params)
            .execute()
        )


    def get_by_id(self):
        """Gets a specific workflow by ID in TruSTAR platform.
        You'll need to call to 'set_workflow_id' before calling this method.
        """
        self._raise_if_workflow_id_is_not_set_up()
        return (
            self.create_query(Methods.GET, "/{}".format(self.workflow_guid))
            .set_params(self.payload_params)
            .execute()
        )


    def delete(self):
        """Deletes a specific workflow by ID in TruSTAR platform.
        You'll need to call to 'set_workflow_id' before calling this method.
        """
        self._raise_if_workflow_id_is_not_set_up()
        return (
            self.create_query(Methods.DELETE, "/{}".format(self.workflow_guid))
            .set_params(self.payload_params)
            .execute()
        )


    def update(self):
        """Updates a workflow in TruSTAR platform.
        
        You'll need to call to the following methods calling to this method: 
            - set_name
            - set_workflow_config
            - set_safelist_ids
            - set_workflow_id
        """
        self._raise_if_payload_is_not_set_up()
        self._raise_if_workflow_id_is_not_set_up()
        return (
            self.create_query(Methods.PUT, "/{}".format(self.workflow_guid))
            .set_params(self.payload_params)
            .execute()
        )



from trustar2.base import fluent, ParamsSerializer, Param


@fluent
class BaseHandler(object):

    def __init__(self, config=None):
        self.config = config
        self.payload_params = ParamsSerializer()
        self.query_params = ParamsSerializer()


    def set_payload_param(self, key, value):
        """Adds a new param to set of payload params."""
        param = Param(key=key, value=value)
        self.payload_params.add(param)


    def set_query_param(self, key, value):
        """Adds a new param to set of query params."""
        param = Param(key=key, value=value)
        self.query_params.add(param)

    def set_trustar_config(self, trustar_config):
        self.config = trustar_config

    def _argument_to_list(self, arg):
        iterables = (list, tuple, set)
        return arg if isinstance(arg, iterables) else [arg]
    
    def _argument_to_unique_list(self, arg):
        return list(set(self._argument_to_list(arg)))



from .api_client import ApiClient
from .base import Param, fluent


@fluent
class Query:

    def __init__(self, config, endpoint, method, params=None, query_string=None):
        self.config = config
        self.endpoint = endpoint
        self.method = method
        self.params = params
        self.query_string = query_string
        self.stop = False
        self.api = ApiClient(self.config)

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def set_query_string(self, query_string):
        self.query_string = query_string

    def set_params(self, params):
        self.params = params

    def _update_cursor(self, response):
        if response["responseMetadata"].get("nextCursor", "") != "":
            cursor = Param("cursor", response["responseMetadata"]["nextCursor"])
            self.params.add(cursor)
        else:
            self.stop = True

    def _update_page(self, response):
        if response["hasNext"]:
            currentPage = int(response.get("pageNumber", 0))
            pageNumber = Param("pageNumber", currentPage + 1)
            self.params.add(pageNumber)
        else:
            self.stop = True

    def _update_params_from_response(self, response):
        if "responseMetadata" in response.keys():
            self._update_cursor(response)
        elif "hasNext" in response.keys():
            self._update_page(response)

    def next(self):
        if not self.stop:
            result = self.api.fetch(self, use_empty_payload=True)
            self._update_params_from_response(result.json())
            return result
        else:
            raise StopIteration

    def execute(self, use_empty_payload=False):
        return self.api.fetch(self, use_empty_payload)



from .base import Base
from trustar2.base import fluent
from trustar2.trustar_enums import WorkflowDestinations, ObservableTypes

MIN_WEIGHT = 1
MAX_WEIGHT = 5


@fluent
class WorkflowConfig(Base):

    def __init__(self, workflow_type="INDICATOR_PRIORITIZATION"):
        self.workflow_type = workflow_type
        self.workflow_source = []
        self.workflow_destination = []
        self.priority_scores = []
        self.observable_types = []


    def _get_source_config_obj_from_tuple(self, source_config):
        return WorkflowSourceConfig(*source_config)


    def _get_source_config_obj_from_dict(self, source_config):
        enclave_guid = source_config.get("enclave_guid")
        weight = source_config.get("weight")
        if not enclave_guid or not weight:
            raise AttributeError("'enclave_guid' or 'weight' fields were not provided in dict.")

        return WorkflowSourceConfig(enclave_guid, weight)


    def _get_source_config_obj(self, source_config):
        if isinstance(source_config, WorkflowSourceConfig):
            return source_config

        if isinstance(source_config, tuple):
            return self._get_source_config_obj_from_tuple(source_config)

        if isinstance(source_config, dict):
            return self._get_source_config_obj_from_dict(source_config)

        raise AttributeError(
            "Valid types to create a source config are: WorkflowSourceConfig, tuple, dict"
        )


    def _get_destination_config_obj_from_tuple(self, destination_config):
        return WorkflowDestinationConfig(*destination_config)


    def _get_destination_config_obj_from_dict(self, destination_config):
        enclave_guid = destination_config.get("enclave_guid")
        destination_type = destination_config.get("destination_type")
        if not enclave_guid or not destination_type:
            raise AttributeError("'enclave_guid' or 'destination_type' fields were not provided in dict.")

        return WorkflowDestinationConfig(enclave_guid, destination_type)


    def _get_destination_config_obj(self, destination_config):
        if isinstance(destination_config, WorkflowDestinationConfig):
            return destination_config

        if isinstance(destination_config, tuple):
            return self._get_destination_config_obj_from_tuple(destination_config)

        if isinstance(destination_config, dict):
            return self._get_destination_config_obj_from_dict(destination_config)

        raise AttributeError(
            "Valid types to create a destination config are: WorkflowDestinationConfig, tuple, dict"
        )


    def _handle_list(self, source_config_list):
        return [self._get_source_config_obj(source) for source in source_config_list]

    
    def _raise_if_observables_are_not_valid(self, observable_types):
        possible_types = set(ObservableTypes.members())
        if set(observable_types) - possible_types:
            raise AttributeError(
                "Observable Types can only be within the following set: {}".format(possible_types)
            )
            

    def set_source_configs(self, source_config):
        """Sets the source configuration for a workflow config. 

        :param source_config: This can be a single element or a list of one of the following:

            - WorkflowSourceConfig.
            - tuple: With first element an enclave_guid (string) and the second 
              a weight (int between 1 and 5)
            - dict: With 'enclave_guid' and 'weight' fields populated
        """
        if isinstance(source_config, list):
            self.workflow_source.extend(self._handle_list(source_config))

        else: 
            self.workflow_source.append(self._get_source_config_obj(source_config))


    def set_destination_configs(self, destination_config):
        """Sets the destination configuration for a workflow config. 

        :param destination_config: This can be a single element or a list of one of the following:

            - WorkflowDestinationConfig.
            - tuple: With first element an enclave_guid (string) and the second 
              a destination_type (string - QRADAR or ENCLAVE)
            - dict: With 'enclave_guid' and 'destination_type' fields populated
        """
        self.workflow_destination.append(self._get_destination_config_obj(destination_config))


    def set_priority_scores(self, priority_scores):
        """Sets the priority scores for a workflow config. 

        :param priority_scores: has to be a list of strings (BENIGN, LOW, MEDIUM, HIGH).
        """
        self.priority_scores.extend(priority_scores)

    
    def set_observable_types(self, observable_types):
        """Sets the observable types for a workflow config. 

        :param observable_types: has to be a list of strings or enums with valid TruSTAR observable types.
        """
        if not isinstance(observable_types, list):
            raise AttributeError("'observable_types' should be a list")

        self._raise_if_observables_are_not_valid(observable_types)
        self.observable_types.extend(observable_types)


    def serialize(self):
        serialized = super(WorkflowConfig, self).serialize()
        workflow_source = serialized.get("workflowSource")
        workflow_destination = serialized.get("workflowDestination")
        serialized.update({"workflowSource": {"enclaveSourceConfig": workflow_source},
                           "workflowDestination": {"enclaveDestinationConfigs": workflow_destination}})
        return serialized



class WorkflowSourceConfig(Base):

    def __init__(self, enclave_guid, weight):
        self._validate_weight(weight)
        self._validate_enclave_guid(enclave_guid)
        self.enclave_guid = enclave_guid
        self.weight = weight


    def _validate_weight(self, weight):
        if weight > MAX_WEIGHT or weight < MIN_WEIGHT:
            raise AttributeError(
                "Workflow source config weights have to be within {} and {}.".format(MIN_WEIGHT, MAX_WEIGHT)
            )

    
    def _validate_enclave_guid(self, enclave_guid):
        if not isinstance(enclave_guid, str):
            raise AttributeError("Enclave GUID provided must be a string.")



class WorkflowDestinationConfig(Base):

    def __init__(self, enclave_guid, destination_type):
        self._validate_destination_type(destination_type)
        self._validate_enclave_guid(enclave_guid)
        self.enclave_guid = enclave_guid
        self.destination_type = destination_type

    
    def _validate_enclave_guid(self, enclave_guid):
        if not isinstance(enclave_guid, str):
            raise AttributeError("Enclave GUID provided must be a string.")


    def _validate_destination_type(self, destination_type):
        possible_types = list(WorkflowDestinations.members())
        if not destination_type in possible_types:
            raise AttributeError("Destination type must be one of the following: {}".format(possible_types))



from __future__ import absolute_import
import json
from os import name

from .log import get_logger
from trustar2 import SearchIndicator, Submission, Safelist, Account, Workflow, Observables
from trustar2.trustar_enums import TruStarUrls
from .version import __version__

logger = get_logger(__name__)


class TruStar:

    DEFAULTS = {
        "auth_endpoint": TruStarUrls.AUTH_TOKEN.value,
        "api_endpoint": TruStarUrls.API.value,
        "station": TruStarUrls.STATION.value,
        "client_type": "PYTHON_SDK",
        "client_version": __version__,
        "verify": True,
        "retry": True,
        "max_wait_time": 60,
        "proxy": {
            "http": None,
            "https": None,
        }
    }

    def __init__(self, api_key, api_secret, client_metatag, **kwargs):
        self.api_key = api_key
        self.api_secret = api_secret
        self.client_metatag = client_metatag
        self.request_details = self.DEFAULTS.copy()
        self.request_details.update(kwargs)

    def get_proxy(self):
        proxies = {key: self.request_details["proxy"][key] for key in self.request_details["proxy"]
                   if self.request_details["proxy"][key]}
        return proxies

    @classmethod
    def config_from_file(cls, config_file_path, config_role):
        with open(config_file_path, "r") as f:
            config_file = json.load(f)

        config = config_file.get(config_role)
        if not config:
            raise AttributeError(
                "{} role was not found in {} file".format(config_role, config_file_path)
            )

        return cls(**config)

    def indicators(self):
        # TODO add condig
        return SearchIndicator(self)

    def submission(self):
        return Submission(self)

    def safelist(self):
        return Safelist(self)

    def account(self):
        return Account(self)
      
    def workflow(self):
        return Workflow(self)

    def observable(self):
        return Observables(self)



from trustar2.handlers.tags import TagIndicator, TagSubmission, TagObservable
from trustar2.handlers.indicators import SearchIndicator
from trustar2.handlers.safelist import Safelist
from trustar2.handlers.submission import Submission
from trustar2.handlers.account import Account
from trustar2.handlers.workflow import Workflow
from trustar2.handlers.observables import Observables



from __future__ import unicode_literals

from trustar2.query import Query
from trustar2.handlers.base_handler import BaseHandler
from trustar2.base import Methods, fluent


@fluent
class TagBase(BaseHandler):

    def __init__(self, endpoint, config=None):
        super(TagBase, self).__init__(config)
        self.guid = None
        self._url = endpoint

    @property
    def base_url(self):
        return self.config.request_details.get("api_endpoint") + "/{}".format(self._url)

    def _validate_payload(self):
        if self.guid is None:
            raise AttributeError(
                "Id value is required for altering tags of {}".format(self._url)
            )
        
        added_tags = self.payload_params.get("addedTags", [])
        removed_tags = self.payload_params.get("removedTags", [])

        if len(added_tags) == 0 and len(removed_tags) == 0:
            raise AttributeError(
                "Either 'addedTags' or 'removedTags' values are required for altering tags of {}".format(self._url)
            )

        if "enclaveGuid" not in self.payload_params and "enclaveId" not in self.payload_params: 
            raise AttributeError(
                "Enclave id value is required for altering tags on {}".format(self._url)
            )


    @property
    def tag_endpoint(self):
        self._validate_payload()
        return self.base_url + "/{}/alter-tags".format(self.guid)

    def _validate_tags(self, tags):
        iterables = (list, tuple, set)
        if type(tags) not in iterables:
            raise AttributeError(u"addedTags {} should be a list of string values".format(tags))

    def set_added_tags(self, added_tags):
        self._validate_tags(added_tags)
        self.set_payload_param("addedTags", added_tags)


    def set_removed_tags(self, removed_tags):
        self._validate_tags(removed_tags)
        self.set_payload_param("removedTags", removed_tags)


    def set_enclave_id(self, enclave_guid):
        if self._url == "submissions":
            self.set_payload_param("enclaveId", enclave_guid)
        else:
            self.set_payload_param("enclaveGuid", enclave_guid)

    def alter_tags(self):
        return (
            Query(self.config, self.tag_endpoint, Methods.POST)
            .set_params(self.payload_params)
            .execute()
        )


@fluent
class TagIndicator(TagBase):

    def __init__(self, config=None):
        super(TagIndicator, self).__init__("indicators", config)


    def set_indicator_id(self, indicator_id):
        self.guid = indicator_id


@fluent
class TagSubmission(TagBase):

    def __init__(self, config=None):
        super(TagSubmission, self).__init__("submissions", config)


    def set_submission_id(self, submission_id):
        self.guid = submission_id


    def set_id_type_as_external(self, external):
        self.set_payload_param("idType", "EXTERNAL" if external else "INTERNAL")


@fluent
class TagObservable(TagBase):

    def __init__(self, config=None):
        super(TagObservable, self).__init__("observables", config)


    def set_observable_value(self, value):
        self.guid = value
        self.set_payload_param("observableValue", value)

    @property
    def tag_endpoint(self):
        self._validate_payload()
        return "{}{}".format(self.base_url, "/alter-tags")



from __future__ import unicode_literals

import json
import pytest

from trustar2 import Safelist, TruStar
from .resources import safelist_summaries, safelist_details, entities_extraction

TRUSTAR_API = "https://test.trustar.co/api/2.0"
SAFELIST_SUMMARIES = TRUSTAR_API + "/safelist-libraries"
SAFELIST_LIBRARY = "test-library-guid-1"
SAFELIST_DETAILS = SAFELIST_SUMMARIES + "/" + SAFELIST_LIBRARY


@pytest.fixture
def safelist():
    config = TruStar(api_key="xxxx", 
                     api_secret="xxx", 
                     client_metatag="test_env", 
                     api_endpoint=TRUSTAR_API)
    return Safelist(config)


@pytest.fixture
def safelist_summaries_json():
    result = json.loads(safelist_summaries)
    return result


@pytest.fixture
def safelist_details_json():
    result = json.loads(safelist_details)
    return result


@pytest.fixture
def entities_extraction_json():
    result = json.loads(entities_extraction)
    return result


def test_get_safelist_summaries(mocked_request, safelist, safelist_summaries_json):
    mocked_request.get(SAFELIST_SUMMARIES, status_code=200, json=safelist_summaries_json)
    response = safelist.get_safelist_libraries()
    assert response.json() == safelist_summaries_json


def test_get_safelist_details_successfully(mocked_request, safelist, safelist_details_json):
    mocked_request.get(SAFELIST_DETAILS, status_code=200, json=safelist_details_json)
    response = safelist.set_library_guid(SAFELIST_LIBRARY).get_safelist_details()
    assert response.json() == safelist_details_json


def test_get_safelist_details_without_library_guid(safelist):
    with pytest.raises(AttributeError):
        safelist.get_safelist_details()


def test_create_entries_successfully(mocked_request, safelist, safelist_details_json):
    mocked_request.patch(SAFELIST_DETAILS, status_code=200, json=safelist_details_json)
    response = safelist \
                .set_library_guid(SAFELIST_LIBRARY) \
                .set_safelist_entries({
                    "entity": "good-email@test-domain.com", 
                    "type": "EMAIL_ADDRESS"
                }).create_entries()

    assert response.json() == safelist_details_json


def test_create_entries_without_library_guid(safelist):
    with pytest.raises(AttributeError):
        safelist.create_entries()


def test_create_entries_without_setting_entries(safelist):
    with pytest.raises(AttributeError):
        safelist.set_library_guid(SAFELIST_LIBRARY).create_entries()


def test_create_entries_without_all_required_fields(safelist):
    with pytest.raises(AttributeError):
        safelist.set_library_guid(SAFELIST_LIBRARY) \
                .set_safelist_entries({
                    "entity": "good-email@test-domain.com"
                }).create_entries()


def test_create_entries_without_strings(safelist):
    with pytest.raises(AttributeError):
        safelist.set_library_guid(SAFELIST_LIBRARY) \
                .set_safelist_entries({
                    100: "good-email@test-domain.com",
                    "type": "EMAIL_ADDRESS"
                }).create_entries()


def test_create_entries_without_valid_type(safelist):
    with pytest.raises(AttributeError):
        safelist.set_library_guid(SAFELIST_LIBRARY) \
                .set_safelist_entries({
                    "entity": "good-email@test-domain.com",
                    "type": "NOT_A_VALID_TYPE"
                }).create_entries()


def test_create_safelist_library_successfully(mocked_request, safelist, safelist_summaries_json):
    mocked_request.post(SAFELIST_SUMMARIES, status_code=200, json=safelist_summaries_json)
    response = safelist.set_library_name("test-library-name-1").create_safelist()
    assert response.json() == safelist_summaries_json


def test_create_safelist_library_without_name(safelist):
    with pytest.raises(AttributeError):
        safelist.create_safelist()


def test_delete_entry_successfully(mocked_request, safelist):
    entry_to_be_deleted = "entry-guid-1"
    url = SAFELIST_DETAILS + "/" + entry_to_be_deleted
    mocked_request.delete(url, status_code=200)
    response = safelist.set_library_guid(SAFELIST_LIBRARY).delete_entry(entry_to_be_deleted)
    assert response.status_code == 200


def test_delete_entry_without_library_guid(safelist):
    with pytest.raises(AttributeError):
        safelist.delete_entry("entry-guid-1")


def test_delete_safelist_library_successfully(mocked_request, safelist):
    mocked_request.delete(SAFELIST_DETAILS, status_code=200)
    response = safelist.set_library_guid(SAFELIST_LIBRARY).delete_safelist()
    assert response.status_code == 200


def test_delete_safelist_library_without_guid(safelist):
    with pytest.raises(AttributeError):
        safelist.delete_safelist()


def test_extract_terms_successfully(mocked_request, safelist, entities_extraction_json):
    url = safelist.extract_endpoint
    mocked_request.post(url, json=entities_extraction_json)
    text_blob = "IP: 8.8.8.8\nUnstructured text extraction\ngood-email@test-domain.com"
    response = safelist.set_text_to_be_extracted(text_blob).extract_terms()
    assert response.json() == entities_extraction_json


def test_extract_terms_without_setting_text(safelist):
    with pytest.raises(AttributeError):
        safelist.extract_terms()



import pytest

from trustar2.api_client import ApiClient
from trustar2.trustar import TruStar
from trustar2.version import __version__


@pytest.fixture
def api_client(ts):
    return ApiClient(TruStar("API_KEY", "API_SECRET", "TEST_METATAG"))


def test_get_token_successfully(api_client, mocked_request):
    token = api_client._get_token()
    assert token == "TOKEN12345"


def test_get_headers(api_client, mocked_request):
    headers = api_client._get_headers("POST")
    expected_headers = {
        "Authorization": "Bearer TOKEN12345",
        "Client-Metatag": "TEST_METATAG",
        "Client-Type": "PYTHON_SDK",
        "Client-Version": __version__,
        "Content-Type": "application/json",
    }
    assert headers == expected_headers


def test_token_is_not_expired(api_client, mocker):
    mock_response = mocker.Mock(status_code=200)
    expired = api_client._token_is_expired(mock_response)
    assert not expired


def test_token_is_expired(api_client, mocker):
    mock_response = mocker.Mock(
        status_code=400,
        json=lambda: {"error_description": "Expired oauth2 access token"},
    )
    expired = api_client._token_is_expired(mock_response)
    assert expired


def test_sleep_wait_time_lower_max_wait_time(api_client, mocker):
    mock_response = mocker.Mock(
        json=lambda: {"waitTime": 500},
    )
    keep_trying = api_client._sleep(mock_response)
    assert keep_trying


def test_sleep_wait_time_higher_max_wait_time(api_client, mocker):
    mock_response = mocker.Mock(
        json=lambda: {"waitTime": 65000},
    )
    keep_trying = api_client._sleep(mock_response)
    assert not keep_trying



from trustar2.query import Query
from trustar2.base import fluent, Methods
from trustar2.handlers.base_handler import BaseHandler


@fluent
class Account(BaseHandler):


    def __init__(self, config=None):
        super(Account, self).__init__(config)


    @property
    def endpoint(self):
        return self.config.request_details.get("api_endpoint")
        

    def create_query(self, method, specific_endpoint=""):
        """Returns a new instance of a Query object according config, endpoint and method."""
        endpoint = self.endpoint + specific_endpoint
        return Query(self.config, endpoint, method)


    def ping(self):
        """Tests connectiviy against TruSTAR API."""
        return self.create_query(Methods.GET, specific_endpoint="/ping").execute()


    def get_enclaves(self):
        """Returns all user enclaves with according permissions."""
        return self.create_query(Methods.GET, specific_endpoint="/enclaves").execute()



from __future__ import unicode_literals

import pytest
from trustar2.models import Indicator, Entity


@pytest.fixture
def indicator():
    return Indicator("URL", "www.badurl.com")


def test_set_related_observables(indicator):
    related_observable_email = Entity.observable("EMAIL_ADDRESS", "bob@gmail.com")
    indicator.set_related_observables(related_observable_email)
    assert related_observable_email in indicator.related_observables


def test_set_attributes(indicator):
    attributes = Entity.attribute("THREAT_ACTOR", "ActorName")
    indicator.set_attributes(attributes)
    assert attributes in indicator.attributes


def test_set_valid_to(indicator):
    indicator.set_valid_to(1604510497)
    value = indicator.observable.params.get("validTo")
    assert value == 1604510497


def test_set_valid_from(indicator):
    indicator.set_valid_from(1604510497)
    value = indicator.observable.params.get("validFrom")
    assert value == 1604510497


def test_set_malicious_score(indicator):
    indicator.set_malicious_score("HIGH")
    value = indicator.observable.params.get("maliciousScore")
    assert value == "HIGH"


def test_set_confidence_score(indicator):
    indicator.set_confidence_score("HIGH")
    value = indicator.observable.params.get("confidenceScore")
    assert value == "HIGH"

 
def test_set_properties(indicator):
    properties = {"myPropKey": "myPropValue"}
    indicator.set_properties(properties)
    value = indicator.observable.params.get("properties")
    assert value == properties


def test_set_properties_with_more_than_twenty_elements(indicator):
    properties = {"prop" + str(i): "value" + str(i) for i in range(21)}
    with pytest.raises(AttributeError):
        indicator.set_properties(properties)


def test_set_properties_with_element_type_not_a_string(indicator):
    properties = {"myProperty": 100}
    with pytest.raises(AttributeError):
        indicator.set_properties(properties)


def test_set_tags(indicator):
    tag = ["importantTag"]
    indicator.set_tags(tag)
    assert "importantTag" in indicator.tags



import json
import pytest

from trustar2 import Account, TruStar
from tests.unit.resources import enclaves


URL = "https://api.trustar.co/api/2.0"


@pytest.fixture
def account(ts):
    return Account(ts)


@pytest.fixture
def enclaves_response():
    return json.loads(enclaves)


def test_ping_successfully(account, mocked_request):
    mocked_request.get(URL + "/ping", status_code=200, text="pong\n")
    response = account.ping()
    assert response.status_code == 200
    assert response.text == "pong\n"


def test_get_enclaves_successfully(account, mocked_request, enclaves_response):
    mocked_request.get(URL + "/enclaves", json=enclaves_response)
    response = account.get_enclaves()
    assert response.status_code == 200
    assert response.json() == enclaves_response



from setuptools import setup, find_packages
from glob import glob

# read version
version_globals = {}
with open("trustar2/version.py") as fp:
    exec(fp.read(), version_globals)
version = version_globals['__version__']

setup(
    name='trustar2',
    packages=find_packages(exclude=("tests",)),
    version=version,
    author='TruSTAR Technology, Inc.',
    author_email='support@trustar.co',
    url='https://github.com/trustar/trustar-sdk2-proto/',
    download_url='https://github.com/trustar/trustar-sdk2-proto/tarball/%s' % version,
    description='Python SDK2 for the TruSTAR REST API',
    license='MIT',
    install_requires=['json_log_formatter',
                      'future',
                      'python-dateutil',
                      'pytz',
                      'requests',
                      'configparser',
                      'dateparser',
                      'unicodecsv',
                      'tzlocal',
                      'PyYAML',
                      'six'
                      ],
    include_package_data=True,
    use_2to3=True
)



import json
import pytest

from tests.unit.resources import serialized_workflow_config
from trustar2.models import WorkflowConfig, WorkflowSourceConfig, WorkflowDestinationConfig


TEST_ENCLAVE = "test-enclave-id"


@pytest.fixture
def wf_src_config():
    return WorkflowSourceConfig(TEST_ENCLAVE, 3)


@pytest.fixture
def wf_dst_config():
    return WorkflowDestinationConfig(TEST_ENCLAVE, "ENCLAVE")


@pytest.fixture
def wf_config():
    return WorkflowConfig()


@pytest.fixture
def serialized_wf_config():
    return json.loads(serialized_workflow_config)


def test_workflow_source_config_serialization(wf_src_config):
    serialized = wf_src_config.serialize()
    assert serialized == {"enclaveGuid": TEST_ENCLAVE, "weight": 3}


def test_workflow_destination_config_serialization(wf_dst_config):
    serialized = wf_dst_config.serialize()
    assert serialized == {"enclaveGuid": TEST_ENCLAVE, "destinationType": "ENCLAVE"}


def test_workflow_source_serialization_with_invalid_weight():
    with pytest.raises(AttributeError):
        WorkflowSourceConfig(TEST_ENCLAVE, 6)


def test_workflow_destination_serialization_with_invalid_type():
    with pytest.raises(AttributeError):
        WorkflowDestinationConfig(TEST_ENCLAVE, "INVALID-TYPE")


def test_configs_with_invalid_enclave_id():
    with pytest.raises(AttributeError):
        WorkflowSourceConfig(123456789, 5) # enclave_guid not a string

    with pytest.raises(AttributeError):
        WorkflowDestinationConfig(123456789, "ENCLAVE") # enclave_guid not a string


def test_workflow_config_serialization(wf_config, wf_src_config, wf_dst_config, serialized_wf_config):
    wf_config.set_observable_types(["URL", "IP4", "IP6", "SHA256"])
    wf_config.set_priority_scores(["MEDIUM", "HIGH"])
    wf_config.set_source_configs(wf_src_config) # Using obj
    wf_config.set_source_configs([("test-enclave-id2", 3), ("test-enclave-id3", 1)]) # Using list of tuples
    wf_config.set_source_configs([{"enclave_guid": "test-enclave-id4", "weight": 5}]) # Using dict
    wf_config.set_destination_configs(wf_dst_config)
    assert wf_config.serialize() == serialized_wf_config


def test_workflow_config_set_invalid_obs_types(wf_config):
    with pytest.raises(AttributeError):
        wf_config.set_observable_types(["INVALID_TYPE"])

    with pytest.raises(AttributeError):
        wf_config.set_observable_types("URL") # has to be a list


def test_workflow_config_set_invalid_src_and_dst_config_types(wf_config):
    with pytest.raises(AttributeError):
        wf_config.set_source_configs("STRING-IS-NOT-VALID")

    with pytest.raises(AttributeError):
        wf_config.set_destination_configs(1234567890) # int is not valid


def test_workflow_config_with_missing_properties_in_src_config(wf_config):
    with pytest.raises(AttributeError):
        wf_config.set_source_configs({"enclaveGuid": TEST_ENCLAVE}) # missing weight

    with pytest.raises(AttributeError):
        wf_config.set_source_configs({"weight": 3}) # missing enclave_guid


def test_workflow_config_with_missing_properties_in_dst_config(wf_config):
    with pytest.raises(AttributeError):
        wf_config.set_destination_configs({"enclaveGuid": TEST_ENCLAVE}) # missing destination_type

    with pytest.raises(AttributeError):
        wf_config.set_destination_configs({"destination_type": "QRADAR"}) # missing enclave_guid



from __future__ import unicode_literals

from .base import Base
from trustar2.base import fluent, ParamsSerializer, Param, get_timestamp
from trustar2.trustar_enums import AttributeTypes, ObservableTypes



@fluent
class Entity(Base):

    def __init__(self, validator, entity_type, value, alias='entity'):
        self.params = ParamsSerializer()
        if entity_type not in validator.members():
            raise AttributeError(
                "Attribute type should be in the following: {}".format(
                    list(validator.members())
                ))
        self.key = alias
        self.set_custom_param(self.key, {"value": value, "type": entity_type})

    @classmethod
    def attribute(cls, entity_type, value):
        return Entity(AttributeTypes, entity_type, value)

    @classmethod
    def observable(cls, entity_type, value):
        return Entity(ObservableTypes, entity_type, value)

    def set_valid_from(self, valid_from):
        if not isinstance(valid_from, int):
            valid_from = get_timestamp(valid_from)

        self.set_custom_param("validFrom", valid_from)

    def set_valid_to(self, valid_to):
        if not isinstance(valid_to, int):
            valid_to = get_timestamp(valid_to)
        self.set_custom_param("validTo", valid_to)

    def set_confidence_score(self, confidence_score):
        # TODO validations
        self.set_custom_param("confidenceScore", confidence_score)

    def set_malicious_score(self, malicious_score):
        self.set_custom_param("maliciousScore", malicious_score)

    def set_properties(self, properties):
        if len(properties) > 20:
            raise AttributeError("properties field can not have more than 20 elements")

        for k, v in properties.items():
            if not isinstance(k, type("")) or not isinstance(v, type("")): # py2 / py3 compatibility
                raise AttributeError("Both key and value of the properties should be strings.")

        self.set_custom_param("properties", properties)

    def set_custom_param(self, key, value):
        param = Param(key=key, value=value)
        self.params.add(param)

    def serialize(self):
        return self.params.serialize()



from __future__ import unicode_literals

import json
import pytest

from trustar2.models import Entity
from trustar2 import SearchIndicator
from trustar2.trustar_enums import ObservableTypes
from .resources import indicators_example_request


@pytest.fixture
def search_indicator(ts):
    return SearchIndicator(ts)


def test_search_indicators_is_empty(search_indicator):
    assert len(search_indicator.payload_params) == 0


def test_set_priority_scores(search_indicator):
    scores = [1, 2, 3]
    search_indicator.set_priority_scores(scores)
    assert len(search_indicator.payload_params) == 1
    assert search_indicator.payload_params.get("priorityScores") == scores


def test_set_priority_scores_fail(search_indicator):
    scores = [1, 2, 4]
    with pytest.raises(AttributeError):
        search_indicator.set_priority_scores(scores)
    assert len(search_indicator.payload_params) == 0


@pytest.mark.parametrize(
    "obs_types",
    [
        ["URL", "MD5", "IP4"],
        [ObservableTypes.URL, ObservableTypes.MD5, ObservableTypes.IP4],
    ],
)
def test_set_observable_types(search_indicator, obs_types):
    expected_result = ["URL", "MD5", "IP4"]
    search_indicator.set_observable_types(obs_types)
    assert len(search_indicator.payload_params) == 1
    assert search_indicator.payload_params.get("types") == expected_result


def test_fail_set_observable_types(search_indicator):
    with pytest.raises(AttributeError):
        search_indicator.set_observable_types(["INVALID", "URL"])
    assert len(search_indicator.payload_params) == 0


@pytest.mark.parametrize(
    "attributes",
    [[{"type": "MALWARE", "value": "ATTRIBUTE"}], [Entity.attribute("MALWARE", "ATTRIBUTE")]],
)
def test_set_attributes(search_indicator, attributes):
    expected_result = [{"type": "MALWARE", "value": "ATTRIBUTE"}]
    search_indicator.set_attributes(attributes)
    assert len(search_indicator.payload_params) == 1
    assert search_indicator.payload_params.get("attributes") == expected_result


@pytest.mark.parametrize(
    "attributes",
    [
        {"type": "INVALID", "value": "ATTRIBUTE"},
        [{"type": "INVALID", "value": "ATTRIBUTE"}],
    ],
)
def test_fail_attributes(search_indicator, attributes):
    with pytest.raises(AttributeError):
        search_indicator.set_attributes(attributes)
    assert len(search_indicator.payload_params) == 0


@pytest.mark.parametrize(
    "observables",
    [[{"type": "URL", "value": "RELATED_OBS"}], [Entity.observable("URL", "RELATED_OBS")]],
)
def test_set_related_observable(search_indicator, observables):
    expected_result = [{"type": "URL", "value": "RELATED_OBS"}]
    search_indicator.set_related_observables(observables)
    assert len(search_indicator.payload_params) == 1
    assert search_indicator.payload_params.get("relatedObservables") == expected_result


@pytest.mark.parametrize(
    "observables",
    [
        {"type": "URL", "value": "RELATED_OBS"},
        [{"type": "INVALID", "value": "RELATED_OBS"}],
    ],
)
def test_set_related_observable_fail(search_indicator, observables):
    with pytest.raises(AttributeError):
        search_indicator.set_related_observables(observables)
    assert len(search_indicator.payload_params) == 0


def test_query_will_not_work_due_to_invalid_dates(search_indicator):
    search_indicator.set_from("1 day ago")
    search_indicator.set_to("2 days ago")
    with pytest.raises(AttributeError):
        search_indicator.search()


def test_ok_query(search_indicator):
    attribute = [{"type": "THREAT_ACTOR", "value": "BAD PANDA"}]
    p = ("cursor", "eyJwYWdlTnVtYmVyIjoxLCJwYWdlU2l6ZSI6Miwib2Zmc2V0Ijo0fQ==")
    q = (
        search_indicator.set_query_term("/Users/mknopf/code/test.sh")
        .set_enclave_ids("3a93fab3-f87a-407a-9376-8eb3fae99b4e")
        .set_priority_scores([3])
        .set_observable_types([ObservableTypes.SOFTWARE.name])
        .set_from(1596607968000)
        .set_to(1598308171000)
        .set_sort_column("PROCESSED_AT")
        .set_attributes(attribute)
        .set_payload_param(*p)
        .search()
    )
    assert q.params.serialize() == json.loads(indicators_example_request)
